from protocols import events
from protocols.protocol import ClinicalQualityMeasure
from protocols.value_set.v2021 import (
    AcuteInpatient, AnnualWellnessVisit, CareServicesInLongTermResidentialFacility,
    DementiaMedications, Diabetes, DiabeticRetinopathy, Ed, EncounterInpatient, Ethnicity,
    FrailtyDevice, FrailtyDiagnosis, FrailtyEncounter, FrailtySymptom, HomeHealthcareServices,
    HospiceCareAmbulatory, NonacuteInpatient, NursingFacilityVisit, Observation, OfficeVisit,
    OncAdministrativeSex, OphthalmologicalServices, Outpatient, Payer,
    PreventiveCareServicesEstablishedOfficeVisit18AndUp,
    PreventiveCareServicesInitialOfficeVisit18AndUp, Race, RetinalOrDilatedEyeExam)


class ClinicalQualityMeasure131v9(ClinicalQualityMeasure):
    """
    Diabetes: Eye Exam

    Description: Percentage of patients 18-75 years of age with diabetes and an active diagnosis of
    retinopathy overlapping the measurement period who had a retinal or dilated eye exam by an eye
    care professional during the measurement period or diabetics with no diagnosis of retinopathy
    overlapping the measurement period who had a retinal or dilated eye exam by an eye care
    professional during the measurement period or in the 12 months prior to the measurement period

    Definition: None

    Rationale: As the seventh leading cause of death in the U.S., diabetes kills approximately
    79,500 people a year and affects more than 30 million Americans (9.4 percent of the U.S.
    population) (Centers for Disease Control and Prevention [CDC], 2017a, 2017b). Diabetes is a
    long-lasting disease marked by high blood glucose levels, resulting from the body's inability
    to produce or use insulin properly (CDC, 2019). People with diabetes are at increased risk of
    serious health complications including vision loss, heart disease, stroke, kidney failure,
    amputation of toes, feet or legs, and premature death (CDC, 2016).

    In 2017, diabetes cost the U.S. an estimated $327 billion: $237 billion in direct medical costs
    and $90 billion in reduced productivity. This is a 34 percent increase from the estimated $245
    billion spent on diabetes in 2012 (American Diabetes Association, 2018).

    Diabetic retinopathy is progressive damage to the small blood vessels in the retina that may
    result in loss of vision. It is the leading cause of blindness in adults between 20-74 years of
    age. Approximately 4.1 million adults are affected by diabetic retinopathy (CDC, 2015).

    Guidance: Only patients with a diagnosis of Type 1 or Type 2 diabetes should be included in the
    denominator of this measure; patients with a diagnosis of secondary diabetes due to another
    condition should not be included.

    The eye exam must be performed by an ophthalmologist or optometrist.

    This eCQM is a patient-based measure.

    This version of the eCQM uses QDM version 5.5. Please refer to the eCQI resource center
    (https://ecqi.healthit.gov/qdm) for more information on the QDM.

    More information: https://ecqi.healthit.gov/ecqm/measures/cms131v9
    """

    title = 'Diabetes: Eye Exam'

    identifiers = ['CMS131v9']

    types = ['CQM']

    responds_to_event_types = [
        events.HEALTH_MAINTENANCE,
    ]

    authors = [
        'National Committee for Quality Assurance',
    ]

    references = [
        'American Diabetes Association. (2018). Economic costs of diabetes in the U.S. in 2017. Diabetes Care, 41, 917-928. Retrieved from http://care.diabetesjournals.org/content/early/2018/03/20/dci18-0007',
        'American Diabetes Association. (2019). 11. Microvascular complications and foot care: Standards of medical care in diabetes—2019. Diabetes Care, 42(Suppl. 1), S 124-S138. https://doi.org/10.2337/dc19-S011',
        'Centers for Disease Control and Prevention. (2017a). Health, United States, 2016: With chartbook on long-term trends in health. Retrieved from https://www.cdc.gov/nchs/data/hus/hus16.pdf',
        'Centers for Disease Control and Prevention. (2017b). National diabetes statistics report, 2017. Atlanta, GA: U.S. Department of Health and Human Services, CDC. Retrieved from https://www.cdc.gov/diabetes/pdfs/data/statistics/national-diabetes-statistics-report.pdf',
        'Centers for Disease Control and Prevention. (2019). About diabetes. Retrieved from https://www.cdc.gov/diabetes/basics/diabetes.html',
        'Centers for Disease Control and Prevention. (2016). At a glance 2016: Diabetes—Working to reverse the U.S. epidemic. Atlanta, GA: Author. Retrieved from https://upcap.org/admin/wp-content/uploads/2016/06/Diabetes-at-a-Glance.pdf',
        'Centers for Disease Control and Prevention. (2015). Common eye disorders: Diabetic retinopathy. Retrieved from https://www.cdc.gov/visionhealth/basics/ced/index.html',
    ]

    funding_source = ''

    def in_initial_population(self):
        """
        Initial population: Patients 18-75 years of age with diabetes with a visit during the
        measurement period
        """
        pass

    def in_denominator(self):
        """
        Denominator: Equals Initial Population

        Exclusions: Exclude patients whose hospice care overlaps the measurement period.

        Exclude patients 66 and older who are living long term in an institution for more than 90
        consecutive days during the measurement period.

        Exclude patients 66 and older with advanced illness and frailty because it is unlikely that
        patients will benefit from the services being measured.

        Exceptions: None
        """
        return self.in_initial_population()

    def in_numerator(self):
        """
        Numerator: Patients with an eye screening for diabetic retinal disease. This includes
        diabetics who had one of the following:
        •Diabetic with a diagnosis of retinopathy that overlaps the measurement period and a
        retinal or dilated eye exam by an eye care professional in the measurement period
        •Diabetic with no diagnosis of retinopathy overlapping the measurement period and a retinal
        or dilated eye exam by an eye care professional in the measurement period or the year prior
        to the measurement period

        Exclusions: Not Applicable
        """
        pass


    def compute_results(self):
        """
        Clinical recommendation: American Diabetes Association (2019):

        - Adults with type 1 diabetes should have an initial dilated and comprehensive eye
        examination by an ophthalmologist or optometrist within 5 years after the onset of
        diabetes. (Level of evidence: B)

        - Patients with type 2 diabetes should have an initial dilated and comprehensive eye
        examination by an ophthalmologist or optometrist at the time of the diabetes diagnosis.
        (Level of evidence: B)

        -If there is no evidence of retinopathy for one or more annual eye exam and glycemia is
        well controlled, then exams every 1–2 years may be considered. If any level of diabetic
        retinopathy is present, subsequent dilated retinal examinations should be repeated at least
        annually by an ophthalmologist or optometrist. If retinopathy is progressing or sight
        threatening, then examinations will be required more frequently. (Level of evidence: B)
        """
        pass



from ..value_set import ValueSet


class HospiceCareAmbulatory(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of interventions to identify patients receiving hospice care outside of a hospital or long term care facility.

    **Data Element Scope:** This value set may use a model element related to Procedure or Intervention.

    **Inclusion Criteria:** Includes concepts that represent a procedure or intervention for hospice care.

    **Exclusion Criteria:** Excludes concepts that represent palliative care or comfort measures.

    ** Used in:** CMS90v11, CMS134v10, CMS165v10, CMS146v10, CMS124v10, CMS139v10, CMS154v10, CMS56v10, CMS74v11, CMS75v10, CMS137v10, CMS136v11, CMS128v10, CMS122v10, CMS153v10, CMS66v10, CMS130v10, CMS155v10, CMS127v10, CMS117v10, CMS131v10, CMS156v10, CMS125v10
    """

    VALUE_SET_NAME = 'Hospice care ambulatory'
    OID = '2.16.840.1.113762.1.4.1108.15'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    SNOMEDCT = {
        '385763009',  # Hospice care (regime/therapy)
        '385765002',  # Hospice care management (procedure)
    }


class PalliativeCareIntervention(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for palliative care interventions.

    **Data Element Scope:** This value set may use a model element related to Intervention.

    **Inclusion Criteria:** Includes concepts that represent palliative care interventions, including procedures and regime/therapy provided as part of palliative care services.

    **Exclusion Criteria:** Excludes concepts that represent an intervention for hospice.

    ** Used in:** CMS134v10, CMS165v10, CMS131v10, CMS122v10, CMS124v10, CMS156v10, CMS125v10, CMS130v10
    """

    VALUE_SET_NAME = 'Palliative Care Intervention'
    OID = '2.16.840.1.113883.3.464.1003.198.12.1135'
    DEFINITION_VERSION = '20210224'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    SNOMEDCT = {
        '103735009',  # Palliative care (regime/therapy)
        '105402000',  # Visit of patient by chaplain during palliative care (regime/therapy)
        '395669003',  # Specialist palliative care treatment (regime/therapy)
        '395670002',  # Specialist palliative care treatment - inpatient (regime/therapy)
        '395694002',  # Specialist palliative care treatment - daycare (regime/therapy)
        '395695001',  # Specialist palliative care treatment - outpatient (regime/therapy)
        '443761007',  # Anticipatory palliative care (regime/therapy)
        '1841000124106',  # Palliative care medication review (procedure)
        '433181000124107',  # Documentation of palliative care medication action plan (procedure)
    }


class DialysisEducation(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for dialysis education.

    **Data Element Scope:** This value set may use a model element related to Intervention.

    **Inclusion Criteria:** Includes concepts that represent dialysis education.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS134v10
    """

    VALUE_SET_NAME = 'Dialysis Education'
    OID = '2.16.840.1.113883.3.464.1003.109.12.1016'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    SNOMEDCT = {
        '28812006',  # Hemodialysis education (procedure)
        '59596005',  # Hemodialysis education at home (procedure)
        '66402002',  # Peritoneal dialysis education (procedure)
        '385972005',  # Dialysis care education (procedure)
    }


class OtherServicesRelatedToDialysis(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for services related to dialysis.

    **Data Element Scope:** This value set may use a model element related to Intervention.

    **Inclusion Criteria:** Includes concepts that represent an intervention related to dialysis, including care planning, catheter maintenance and care assessment and management.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS134v10
    """

    VALUE_SET_NAME = 'Other Services Related to Dialysis'
    OID = '2.16.840.1.113883.3.464.1003.109.12.1015'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    SNOMEDCT = {
        '3257008',  # Empty and measure peritoneal dialysis fluid (procedure)
        '73257006',  # Peritoneal dialysis catheter maintenance (procedure)
        '233591003',  # Hemoperfusion (procedure)
        '385970002',  # Dialysis care assessment (procedure)
        '385971003',  # Dialysis care (regime/therapy)
        '385973000',  # Dialysis care management (procedure)
        '406168002',  # Dialysis access maintenance (regime/therapy)
        '717738008',  # Peritoneal dialysis care assessment (procedure)
        '718019005',  # Hemodialysis care management (procedure)
        '718308002',  # Peritoneal dialysis care (regime/therapy)
        '718330001',  # Hemodialysis care (regime/therapy)
        '718331002',  # Hemodialysis care assessment (procedure)
        '251000124108',  # Dialysis care plan (regime/therapy)
        '311000124103',  # Peritoneal dialysis care plan (regime/therapy)
    }


class AlcoholAndDrugDependenceTreatment(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for encounters for alcohol and drug dependence treatment.

    **Data Element Scope:** This value set may use a model element related to Intervention.

    **Inclusion Criteria:** Includes concepts that represent an encounter for alcohol and drug abuse and dependence treatment.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS137v10
    """

    VALUE_SET_NAME = 'Alcohol and Drug Dependence Treatment'
    OID = '2.16.840.1.113883.3.464.1003.106.12.1005'
    DEFINITION_VERSION = '20210220'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    HCPCSLEVELII = {
        'G2067',  # Medication assisted treatment, methadone; weekly bundle including dispensing and/or administration, substance use counseling, individual and group therapy, and toxicology testing, if performed (provision of the services by a medicare-enrolled opioid treatment program)
        'G2068',  # Medication assisted treatment, buprenorphine (oral); weekly bundle including dispensing and/or administration, substance use counseling, individual and group therapy, and toxicology testing if performed (provision of the services by a medicare-enrolled opioid treatment program)
        'G2069',  # Medication assisted treatment, buprenorphine (injectable); weekly bundle including dispensing and/or administration, substance use counseling, individual and group therapy, and toxicology testing if performed (provision of the services by a medicare-enrolled opioid treatment program)
        'G2070',  # Medication assisted treatment, buprenorphine (implant insertion); weekly bundle including dispensing and/or administration, substance use counseling, individual and group therapy, and toxicology testing if performed (provision of the services by a medicare-enrolled opioid treatment program)
        'G2071',  # Medication assisted treatment, buprenorphine (implant removal); weekly bundle including dispensing and/or administration, substance use counseling, individual and group therapy, and toxicology testing if performed (provision of the services by a medicare-enrolled opioid treatment program)
        'G2072',  # Medication assisted treatment, buprenorphine (implant insertion and removal); weekly bundle including dispensing and/or administration, substance use counseling, individual and group therapy, and toxicology testing if performed (provision of the services by a medicare-enrolled opioid treatment program)
        'G2073',  # Medication assisted treatment, naltrexone; weekly bundle including dispensing and/or administration, substance use counseling, individual and group therapy, and toxicology testing if performed (provision of the services by a medicare-enrolled opioid treatment program)
        'G2074',  # Medication assisted treatment, weekly bundle not including the drug, including substance use counseling, individual and group therapy, and toxicology testing if performed (provision of the services by a medicare-enrolled opioid treatment program)
        'G2075',  # Medication assisted treatment, medication not otherwise specified; weekly bundle including dispensing and/or administration, substance use counseling, individual and group therapy, and toxicology testing, if performed (provision of the services by a medicare-enrolled opioid treatment program)
        'G2076',  # Intake activities, including initial medical examination that is a complete, fully documented physical evaluation and initial assessment by a program physician or a primary care physician, or an authorized healthcare professional under the supervision of a program physician qualified personnel that includes preparation of a treatment plan that includes the patient's short-term goals and the tasks the patient must perform to complete the short-term goals; the patient's requirements for education, vocational rehabilitation, and employment; and the medical, psycho- social, economic, legal, or other supportive services that a patient needs, conducted by qualified personnel (provision of the services by a medicare-enrolled opioid treatment program); list separately in addition to code for primary procedure
        'G2077',  # Periodic assessment; assessing periodically by qualified personnel to determine the most appropriate combination of services and treatment (provision of the services by a medicare-enrolled opioid treatment program); list separately in addition to code for primary procedure
        'G2080',  # Each additional 30 minutes of counseling in a week of medication assisted treatment, (provision of the services by a medicare-enrolled opioid treatment program); list separately in addition to code for primary procedure
        'G2086',  # Office-based treatment for opioid use disorder, including development of the treatment plan, care coordination, individual therapy and group therapy and counseling; at least 70 minutes in the first calendar month
        'G2087',  # Office-based treatment for opioid use disorder, including care coordination, individual therapy and group therapy and counseling; at least 60 minutes in a subsequent calendar month
        'H0020',  # Alcohol and/or drug services; methadone administration and/or service (provision of the drug by a licensed program)
        'H0033',  # Oral medication administration, direct observation
    }
    SNOMEDCT = {
        '20093000',  # Alcohol rehabilitation and detoxification (regime/therapy)
        '23915005',  # Combined alcohol and drug rehabilitation and detoxification (regime/therapy)
        '24165007',  # Alcoholism counseling (procedure)
        '56876005',  # Drug rehabilitation and detoxification (regime/therapy)
        '60112009',  # Drug addiction counseling (procedure)
        '171047005',  # Drugs of addiction education (procedure)
        '266707007',  # Drug addiction therapy (regime/therapy)
        '310653000',  # Drug addiction therapy using methadone (regime/therapy)
        '313071005',  # Counseling for substance abuse (procedure)
        '370881007',  # Abuse prevention assessment (procedure)
        '370884004',  # Abuse prevention management (procedure)
        '385989002',  # Substance use therapy (regime/therapy)
        '386448003',  # Substance use prevention (procedure)
        '386449006',  # Substance use treatment: alcohol withdrawal (regime/therapy)
        '386450006',  # Substance use treatment: drug withdrawal (regime/therapy)
        '386451005',  # Substance use treatment: overdose (regime/therapy)
        '408933008',  # Substance abuse prevention (procedure)
        '408934002',  # Substance abuse prevention assessment (procedure)
        '408935001',  # Substance abuse prevention education (procedure)
        '408936000',  # Substance abuse prevention management (procedure)
        '408941008',  # Drug abuse prevention (procedure)
        '408942001',  # Drug abuse prevention assessment (procedure)
        '408943006',  # Drug abuse prevention education (procedure)
        '408944000',  # Drug abuse prevention management (procedure)
        '408945004',  # Alcohol abuse prevention (procedure)
        '408947007',  # Alcohol abuse prevention education (procedure)
        '408948002',  # Alcohol abuse prevention management (procedure)
        '410419007',  # Substance use surveillance (regime/therapy)
        '413473000',  # Counseling about alcohol consumption (procedure)
        '423416000',  # Substance use cessation case management (procedure)
        '424148004',  # Substance use cessation surveillance (regime/therapy)
        '424407005',  # Substance use cessation education, guidance, and counseling (procedure)
        '424589009',  # Substance use treatment: cessation (regime/therapy)
        '426928008',  # Prevention of opioid abuse (procedure)
        '707166002',  # Alcohol reduction program (regime/therapy)
        '720174008',  # Drug harm reduction program (regime/therapy)
        '720175009',  # Alcohol harm reduction program (regime/therapy)
        '720176005',  # Alcohol relapse prevention program (regime/therapy)
        '720177001',  # Drug relapse prevention program (regime/therapy)
        '737363002',  # Alcohol abuse surveillance (regime/therapy)
        '792901003',  # Drug addiction therapy using buprenorphine (regime/therapy)
        '792902005',  # Drug addiction therapy using buprenorphine and naloxone (regime/therapy)
        '429291000124102',  # Alcohol brief intervention (procedure)
    }


class PsychVisitPsychotherapy(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for encounters for psychotherapy visits.

    **Data Element Scope:** This value set may use a model element related to Encounter.

    **Inclusion Criteria:** Includes concepts that represent an encounter for individual psychotherapy services.

    **Exclusion Criteria:** Excludes concepts that represent an encounter for group psychotherapy or family psychotherapy visits.

    ** Used in:** CMS137v10
    """

    VALUE_SET_NAME = 'Psych Visit - Psychotherapy'
    OID = '2.16.840.1.113883.3.526.3.1496'
    DEFINITION_VERSION = '20190315'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    CPT = {
        '90832',  # Psychotherapy, 30 minutes with patient
        '90834',  # Psychotherapy, 45 minutes with patient
        '90837',  # Psychotherapy, 60 minutes with patient
    }
    SNOMEDCT = {
        '18512000',  # Individual psychotherapy (regime/therapy)
        '38678006',  # Client-centered psychotherapy (regime/therapy)
        '75516001',  # Psychotherapy (regime/therapy)
        '90102008',  # Social psychotherapy (regime/therapy)
        '183381005',  # General psychotherapy (regime/therapy)
        '183382003',  # Psychotherapy - behavioral (regime/therapy)
        '183383008',  # Psychotherapy - cognitive (regime/therapy)
        '302242004',  # Long-term psychodynamic psychotherapy (regime/therapy)
        '304820009',  # Developmental psychodynamic psychotherapy (regime/therapy)
        '304822001',  # Psychodynamic-interpersonal psychotherapy (regime/therapy)
        '314034001',  # Psychodynamic psychotherapy (regime/therapy)
        '401157001',  # Brief solution focused psychotherapy (regime/therapy)
        '443730003',  # Interpersonal psychotherapy (regime/therapy)
    }


class TobaccoUseCessationCounseling(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for tobacco cessation counseling.

    **Data Element Scope:** This value set may use a model element related to Intervention.

    **Inclusion Criteria:** Includes concepts that represent an intervention that may include referral to tobacco cessation-related services or providers, education about the benefits of stopping tobacco use, education about the negative side effects of using tobacco, and monitoring for tobacco cessation.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS138v10
    """

    VALUE_SET_NAME = 'Tobacco Use Cessation Counseling'
    OID = '2.16.840.1.113883.3.526.3.509'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    CPT = {
        '99406',  # Smoking and tobacco use cessation counseling visit; intermediate, greater than 3 minutes up to 10 minutes
        '99407',  # Smoking and tobacco use cessation counseling visit; intensive, greater than 10 minutes
    }
    SNOMEDCT = {
        '171055003',  # Pregnancy smoking education (procedure)
        '185795007',  # Stop smoking monitoring verbal invite (procedure)
        '185796008',  # Stop smoking monitoring telephone invite (procedure)
        '225323000',  # Smoking cessation education (procedure)
        '225324006',  # Smoking effects education (procedure)
        '310429001',  # Smoking monitoring invitation (procedure)
        '315232003',  # Referral to stop-smoking clinic (procedure)
        '384742004',  # Smoking cessation assistance (regime/therapy)
        '395700008',  # Referral to smoking cessation advisor (procedure)
        '702388001',  # Tobacco use cessation education (procedure)
        '710081004',  # Smoking cessation therapy (regime/therapy)
        '711028002',  # Counseling about tobacco use (procedure)
        '713700008',  # Smoking cessation drug therapy (regime/therapy)
        '449841000124108',  # Referral to tobacco use cessation clinic (procedure)
        '449851000124105',  # Referral to tobacco use cessation counselor (procedure)
        '449861000124107',  # Referral to tobacco use cessation counseling program (procedure)
        '449871000124100',  # Referral to tobacco use quit line (procedure)
    }


class CognitiveAssessment(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for assessments performed for the evaluation of cognition.

    **Data Element Scope:** This value set may use a model element related to Intervention.

    **Inclusion Criteria:** Includes concepts that describe assessments for evaluation of cognition.

    **Exclusion Criteria:** Excludes concepts that identify specific standardized tools used to evaluate cognition.

    ** Used in:** CMS149v10
    """

    VALUE_SET_NAME = 'Cognitive Assessment'
    OID = '2.16.840.1.113883.3.526.3.1332'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    SNOMEDCT = {
        '4719001',  # Psychologic cognitive testing and assessment (procedure)
        '113024001',  # Assessment and interpretation of higher cerebral function, cognitive testing (procedure)
    }


class CounselingForNutrition(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for nutrition counseling interventions.

    **Data Element Scope:** This value set may use a model element related to Intervention.

    **Inclusion Criteria:** Includes concepts that represent counseling for nutrition, including medical nutrition therapy, dietetic services, diet education, and weight loss management.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS155v10
    """

    VALUE_SET_NAME = 'Counseling for Nutrition'
    OID = '2.16.840.1.113883.3.464.1003.195.12.1003'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    CPT = {
        '97802',  # Medical nutrition therapy; initial assessment and intervention, individual, face-to-face with the patient, each 15 minutes
        '97803',  # Medical nutrition therapy; re-assessment and intervention, individual, face-to-face with the patient, each 15 minutes
        '97804',  # Medical nutrition therapy; group (2 or more individual(s)), each 30 minutes
    }
    SNOMEDCT = {
        '11816003',  # Diet education (procedure)
        '61310001',  # Nutrition education (procedure)
        '183059007',  # High fiber diet education (procedure)
        '183060002',  # Low residue diet education (procedure)
        '183061003',  # Low fat diet education (procedure)
        '183062005',  # Low cholesterol diet education (procedure)
        '183063000',  # Low salt diet education (procedure)
        '183065007',  # Low carbohydrate diet education (procedure)
        '183066008',  # Low protein diet education (procedure)
        '183067004',  # High protein diet education (procedure)
        '183070000',  # Vegetarian diet education (procedure)
        '183071001',  # Vegan diet education (procedure)
        '226067002',  # Food hygiene education (procedure)
        '266724001',  # Weight-reducing diet education (procedure)
        '275919002',  # Weight loss advised (situation)
        '281085002',  # Sugar-free diet education (procedure)
        '284352003',  # Obesity diet education (procedure)
        '305849009',  # Seen by dietetics service (finding)
        '305850009',  # Seen by community-based dietetics service (finding)
        '305851008',  # Seen by hospital-based dietetics service (finding)
        '306163007',  # Referral to dietetics service (procedure)
        '306164001',  # Referral to community-based dietetics service (procedure)
        '306165000',  # Referral to hospital-based dietetics service (procedure)
        '306626002',  # Discharge from dietetics service (procedure)
        '306627006',  # Discharge from hospital dietetics service (procedure)
        '306628001',  # Discharge from community dietetics service (procedure)
        '313210009',  # Fluid intake education (procedure)
        '370847001',  # Dietary needs education (procedure)
        '386464006',  # Prescribed diet education (procedure)
        '404923009',  # Weight gain advised (situation)
        '408910007',  # Enteral feeding education (procedure)
        '410171007',  # Nutrition care education (procedure)
        '410177006',  # Special diet education (procedure)
        '410200000',  # Weight control education (procedure)
        '429095004',  # Dietary education for weight gain (procedure)
        '431482008',  # Dietary education for competitive athlete (procedure)
        '443288003',  # Lifestyle education regarding diet (procedure)
        '609104008',  # Educated about weight management (situation)
        '698471002',  # Patient advised about weight management (situation)
        '699827002',  # Dietary education about fluid restriction (procedure)
        '699829004',  # High energy diet education (procedure)
        '699830009',  # Food fortification education (procedure)
        '699849008',  # Healthy eating education (procedure)
        '700154005',  # Seen in weight management clinic (finding)
        '700258004',  # Dietary education about vitamin intake (procedure)
        '705060005',  # Diet education about mineral intake (procedure)
        '710881000',  # Education about eating pattern (procedure)
        '428461000124101',  # Referral to nutrition professional (procedure)
        '428691000124107',  # Vitamin K dietary intake education (procedure)
        '441041000124100',  # Counseling about nutrition (procedure)
        '441201000124108',  # Counseling about nutrition using cognitive-behavioral theoretical approach (procedure)
        '441231000124100',  # Counseling about nutrition using health belief model (procedure)
        '441241000124105',  # Counseling about nutrition using social learning theory approach (procedure)
        '441251000124107',  # Counseling about nutrition using transtheoretical model and stages of change approach (procedure)
        '441261000124109',  # Counseling about nutrition using motivational interviewing strategy (procedure)
        '441271000124102',  # Counseling about nutrition using goal setting strategy (procedure)
        '441281000124104',  # Counseling about nutrition using self-monitoring strategy (procedure)
        '441291000124101',  # Counseling about nutrition using problem solving strategy (procedure)
        '441301000124100',  # Counseling about nutrition using social support strategy (procedure)
        '441311000124102',  # Counseling about nutrition using stress management strategy (procedure)
        '441321000124105',  # Counseling about nutrition using stimulus control strategy (procedure)
        '441331000124108',  # Counseling about nutrition using cognitive restructuring strategy (procedure)
        '441341000124103',  # Counseling about nutrition using relapse prevention strategy (procedure)
        '441351000124101',  # Counseling about nutrition using rewards and contingency management strategy (procedure)
        '445291000124103',  # Nutrition-related skills education (procedure)
        '445301000124102',  # Content-related nutrition education (procedure)
        '445331000124105',  # Nutrition-related laboratory result interpretation education (procedure)
        '445641000124105',  # Technical nutrition education (procedure)
    }


class CounselingForPhysicalActivity(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for physical activity counseling.

    **Data Element Scope:** This value set may use a model element related to Intervention.

    **Inclusion Criteria:** Includes concepts that represent counseling or referrals for physical activity, including weight management services.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS155v10
    """

    VALUE_SET_NAME = 'Counseling for Physical Activity'
    OID = '2.16.840.1.113883.3.464.1003.118.12.1035'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    SNOMEDCT = {
        '103736005',  # History and physical examination, sports participation (procedure)
        '183073003',  # Patient advised about exercise (situation)
        '281090004',  # Recommendation to exercise (procedure)
        '304507003',  # Exercise education (procedure)
        '304549008',  # Giving encouragement to exercise (procedure)
        '304558001',  # Reassuring about exercise (procedure)
        '310882002',  # Exercise on prescription (regime/therapy)
        '386291006',  # Exercise promotion: strength training (procedure)
        '386292004',  # Exercise promotion: stretching (procedure)
        '386463000',  # Prescribed activity/exercise education (procedure)
        '390864007',  # Referral for exercise therapy (procedure)
        '390893007',  # Referral to physical activity program (procedure)
        '398636004',  # Physical activity assessment (procedure)
        '398752005',  # Referral to weight maintenance regimen service (procedure)
        '408289007',  # Refer to weight management program (procedure)
        '410200000',  # Weight control education (procedure)
        '410289001',  # Exercises education, guidance, and counseling (procedure)
        '410335001',  # Exercises case management (procedure)
        '429778002',  # Patient given written advice on benefits of physical activity (situation)
        '710849009',  # Assessment of exercise behavior (procedure)
        '435551000124105',  # Counseling about physical activity (procedure)
    }


class PalliativeOrHospiceCare(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of an intervention or procedure to identify patients receiving palliative, comfort or hospice care.

    **Data Element Scope:** This value set may use a model element related to Intervention or Procedure.

    **Inclusion Criteria:** Includes concepts that identify an intervention or procedure for palliative, comfort or hospice care.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS347v5, CMS159v10, CMS69v10
    """

    VALUE_SET_NAME = 'Palliative or Hospice Care'
    OID = '2.16.840.1.113883.3.600.1.1579'
    DEFINITION_VERSION = '20210224'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    SNOMEDCT = {
        '103735009',  # Palliative care (regime/therapy)
        '133918004',  # Comfort measures (regime/therapy)
        '182964004',  # Terminal care (regime/therapy)
        '305284002',  # Admission by palliative care physician (procedure)
        '305381007',  # Admission to palliative care department (procedure)
        '305981001',  # Referral by palliative care physician (procedure)
        '306237005',  # Referral to palliative care service (procedure)
        '306288008',  # Referral to palliative care physician (procedure)
        '385736008',  # Dying care (regime/therapy)
        '385763009',  # Hospice care (regime/therapy)
    }


class DietaryRecommendations(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for recommendations and education for diet and nutrition.

    **Data Element Scope:** This value set may use a model element related to Intervention or Procedure.

    **Inclusion Criteria:** This value set may use a model element related to Intervention or Procedure.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS22v10
    """

    VALUE_SET_NAME = 'Dietary Recommendations'
    OID = '2.16.840.1.113883.3.600.1515'
    DEFINITION_VERSION = '20210220'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    HCPCSLEVELII = {
        'S9452',  # Nutrition classes, non-physician provider, per session
        'S9470',  # Nutritional counseling, dietitian visit
    }
    ICD10CM = {
        'Z71.3',  # Dietary counseling and surveillance
    }
    SNOMEDCT = {
        '11816003',  # Diet education (procedure)
        '61310001',  # Nutrition education (procedure)
        '103699006',  # Patient referral to dietitian (procedure)
        '182922004',  # Dietary regime (regime/therapy)
        '182954008',  # Dietary prophylaxis (regime/therapy)
        '182955009',  # Low carbohydrate diet - prophylaxis (regime/therapy)
        '182956005',  # Low calorie diet - prophylaxis (regime/therapy)
        '182960008',  # Unsaturated fat diet - prophylaxis (regime/therapy)
        '183061003',  # Low fat diet education (procedure)
        '183065007',  # Low carbohydrate diet education (procedure)
        '183070000',  # Vegetarian diet education (procedure)
        '183071001',  # Vegan diet education (procedure)
        '281085002',  # Sugar-free diet education (procedure)
        '284071006',  # Dietary treatment for disorder (regime/therapy)
        '284352003',  # Obesity diet education (procedure)
        '285383009',  # Recommendation to change fruit and nut intake (procedure)
        '289176001',  # Recommendation to change sodium intake (procedure)
        '304491008',  # Dietary education for disorder (procedure)
        '306163007',  # Referral to dietetics service (procedure)
        '361231003',  # Prescribed dietary intake (regime/therapy)
        '370847001',  # Dietary needs education (procedure)
        '386464006',  # Prescribed diet education (procedure)
        '410114009',  # Dietary compliance education (procedure)
        '410171007',  # Nutrition care education (procedure)
        '410177006',  # Special diet education (procedure)
        '410270001',  # Nutritionist education, guidance, and counseling (procedure)
        '413315001',  # Nutrition / feeding management (regime/therapy)
        '418995006',  # Feeding regime (regime/therapy)
        '424753004',  # Dietary management education, guidance, and counseling (procedure)
        '443288003',  # Lifestyle education regarding diet (procedure)
        '699849008',  # Healthy eating education (procedure)
        '715282001',  # Combined healthy eating and physical education program (regime/therapy)
        '770749002',  # Referral for combined healthy eating and physical education program (procedure)
        '1151000175103',  # Dietary Approaches to Stop Hypertension diet (regime/therapy)
        '435771000124106',  # General healthful diet (regime/therapy)
        '437231000124109',  # Sodium modified diet (regime/therapy)
        '437421000124105',  # Decreased sodium diet (regime/therapy)
    }


class FollowUpWithin4Weeks(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for follow-up timing.

    **Data Element Scope:** This value set may use a model element related to Intervention.

    **Inclusion Criteria:** Includes concepts that represent follow-up timing from one day to one month.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS22v10
    """

    VALUE_SET_NAME = 'Follow Up Within 4 Weeks'
    OID = '2.16.840.1.113883.3.526.3.1578'
    DEFINITION_VERSION = '20200307'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    SNOMEDCT = {
        '183617005',  # Follow-up 1 day (finding)
        '183618000',  # Follow-up 2-3 days (finding)
        '183619008',  # Follow-up 4-6 days (finding)
        '183620002',  # Follow-up 1 week (finding)
        '183621003',  # Follow-up 2 weeks (finding)
        '183622005',  # Follow-up 3 weeks (finding)
        '183623000',  # Follow-up 1 month (finding)
    }


class LifestyleRecommendation(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for  lifestyle needs and education related to hypertension.

    **Data Element Scope:** This value set may use a model element related to Procedure or Intervention.

    **Inclusion Criteria:** Includes concepts that represent a procedure or intervention for lifestyle education focused on hypertension.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS22v10
    """

    VALUE_SET_NAME = 'Lifestyle Recommendation'
    OID = '2.16.840.1.113883.3.526.3.1581'
    DEFINITION_VERSION = '20200307'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    SNOMEDCT = {
        '39155009',  # Hypertension education (procedure)
        '313204009',  # Lifestyle education (procedure)
        '443402002',  # Lifestyle education regarding hypertension (procedure)
    }


class RecommendationToIncreasePhysicalActivity(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for recommendations for exercise and nutrition education.

    **Data Element Scope:** This value set may use a model element related to Intervention or Procedure.

    **Inclusion Criteria:** Includes concepts that represent an intervention or procedure for promoting exercise and nutrition regimens.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS22v10
    """

    VALUE_SET_NAME = 'Recommendation to Increase Physical Activity'
    OID = '2.16.840.1.113883.3.600.1518'
    DEFINITION_VERSION = '20190313'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    HCPCSLEVELII = {
        'S9451',  # Exercise classes, non-physician provider, per session
    }
    SNOMEDCT = {
        '281090004',  # Recommendation to exercise (procedure)
        '304507003',  # Exercise education (procedure)
        '304549008',  # Giving encouragement to exercise (procedure)
        '386291006',  # Exercise promotion: strength training (procedure)
        '386292004',  # Exercise promotion: stretching (procedure)
        '386373004',  # Nutrition therapy (regime/therapy)
        '386463000',  # Prescribed activity/exercise education (procedure)
        '410289001',  # Exercises education, guidance, and counseling (procedure)
    }


class ReferralOrCounselingForAlcoholConsumption(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for interventions relevant to alcohol use.

    **Data Element Scope:** This value set may use a model element related to Procedure or Intervention.

    **Inclusion Criteria:** Includes concepts that indicate an intervention for the type of education provided, referral to community service, or rehabilitation center for alcohol use.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS22v10
    """

    VALUE_SET_NAME = 'Referral or Counseling for Alcohol Consumption'
    OID = '2.16.840.1.113883.3.526.3.1583'
    DEFINITION_VERSION = '20200307'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    SNOMEDCT = {
        '24165007',  # Alcoholism counseling (procedure)
        '38670004',  # Referral to alcoholism rehabilitation service (procedure)
        '390857005',  # Referral to community alcohol team (procedure)
        '408947007',  # Alcohol abuse prevention education (procedure)
        '413473000',  # Counseling about alcohol consumption (procedure)
        '417096006',  # Referral to community drug and alcohol team (procedure)
        '431260004',  # Referral to specialist alcohol treatment service (procedure)
    }


class ReferralToPrimaryCareOrAlternateProvider(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for referrals to an alternate or primary care provider.

    **Data Element Scope:** This value set may use a model element related to Intervention.

    **Inclusion Criteria:** Includes concepts that describe an intervention of a referral to an alternate or primary care provider.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS22v10
    """

    VALUE_SET_NAME = 'Referral to Primary Care or Alternate Provider'
    OID = '2.16.840.1.113883.3.526.3.1580'
    DEFINITION_VERSION = '20210220'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    SNOMEDCT = {
        '134403003',  # Urgent referral (procedure)
        '183516009',  # Referral to general medical service (procedure)
        '183561008',  # Referral to general practitioner (procedure)
        '183856001',  # Referral to hypertension clinic (procedure)
        '306206005',  # Referral to service (procedure)
        '306253008',  # Referral to doctor (procedure)
        '306362008',  # Referral to pharmacist (procedure)
        '308470006',  # Referral to general physician (procedure)
        '453641000124107',  # Referral to specialist pharmacist (procedure)
    }


class WeightReductionRecommended(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for discussion, education and  management of weight reduction.

    **Data Element Scope:** This value set may use a model element related to Intervention or Procedure.

    **Inclusion Criteria:** Includes concepts that represent discussion, education and  management of weight reduction.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS22v10
    """

    VALUE_SET_NAME = 'Weight Reduction Recommended'
    OID = '2.16.840.1.113883.3.600.1510'
    DEFINITION_VERSION = '20210220'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    HCPCSLEVELII = {
        'S9449',  # Weight management classes, non-physician provider, per session
    }
    SNOMEDCT = {
        '170795002',  # Follow-up obesity assessment (regime/therapy)
        '266724001',  # Weight-reducing diet education (procedure)
        '268523001',  # Target weight discussed (regime/therapy)
        '284352003',  # Obesity diet education (procedure)
        '388975008',  # Weight reduction consultation (procedure)
        '398752005',  # Referral to weight maintenance regimen service (procedure)
        '408289007',  # Refer to weight management program (procedure)
        '410200000',  # Weight control education (procedure)
        '410201001',  # Weight maintenance regimen management (procedure)
        '445033007',  # Discussion about ideal body weight (procedure)
        '1181000175107',  # Recommendation to lose weight (procedure)
    }


class FollowUpForAdolescentDepression(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for follow-up plans used to document a plan is in place for the treatment of depression that specifically pertains to the adolescent population.

    **Data Element Scope:** This value set may use a model element related to Intervention.

    **Inclusion Criteria:** Includes concepts that represent emotional and coping support as well as mental health management in an attempt to follow up on previously evaluated and diagnosed depression or depressive disorder in adolescents.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS2v11
    """

    VALUE_SET_NAME = 'Follow Up for Adolescent Depression'
    OID = '2.16.840.1.113883.3.526.3.1569'
    DEFINITION_VERSION = '20210220'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    SNOMEDCT = {
        '1555005',  # Brief group psychotherapy (regime/therapy)
        '5694008',  # Crisis intervention with follow-up (regime/therapy)
        '15558000',  # Expressive psychotherapy (regime/therapy)
        '18512000',  # Individual psychotherapy (regime/therapy)
        '28868002',  # Interactive group medical psychotherapy (regime/therapy)
        '75516001',  # Psychotherapy (regime/therapy)
        '76168009',  # Group psychotherapy (regime/therapy)
        '76740001',  # Psychiatric telephone consultation or therapy with patient (procedure)
        '81294000',  # Patient referral for psychotherapy (procedure)
        '88848003',  # Psychiatric follow-up (procedure)
        '91310009',  # Patient follow-up to return when and if necessary (procedure)
        '108313002',  # Family psychotherapy procedure (regime/therapy)
        '228557008',  # Cognitive and behavioral therapy (regime/therapy)
        '229065009',  # Exercise therapy (regime/therapy)
        '372067001',  # Implementation of measures to provide psychological support (regime/therapy)
        '385721005',  # Coping support assessment (procedure)
        '385724002',  # Coping support management (procedure)
        '385725001',  # Emotional support assessment (procedure)
        '385726000',  # Emotional support education (procedure)
        '385727009',  # Emotional support management (procedure)
        '385887004',  # Mental health history taking assessment (procedure)
        '385889001',  # Mental health history taking education (procedure)
        '385890005',  # Mental health history taking management (procedure)
        '386472008',  # Telephone consultation (procedure)
        '401277000',  # Completion of mental health crisis plan (procedure)
        '405780009',  # Dialectical behavior therapy (regime/therapy)
        '410223002',  # Mental health care assessment (procedure)
        '410224008',  # Mental health care education (procedure)
        '410225009',  # Mental health care management (procedure)
        '410226005',  # Mental health promotion assessment (procedure)
        '410227001',  # Mental health promotion education (procedure)
        '410228006',  # Mental health promotion management (procedure)
        '410229003',  # Mental health screening assessment (procedure)
        '410230008',  # Mental health screening education (procedure)
        '410231007',  # Mental health screening management (procedure)
        '410232000',  # Mental health treatment assessment (procedure)
        '410233005',  # Mental health treatment education (procedure)
        '410234004',  # Management of mental health treatment (procedure)
        '425604002',  # Case management follow up (procedure)
        '439141002',  # Discharge by mental health primary care worker (procedure)
    }


class FollowUpForAdultDepression(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for follow-up plans used to document a plan is in place for the treatment of depression specifically pertaining to the adult population.

    **Data Element Scope:** This value set may use a model element related to Intervention.

    **Inclusion Criteria:** Includes concepts that represent emotional and coping support as well as mental health management in an attempt to follow up on previously evaluated and diagnosed depression or depressive disorder in the adults.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS2v11
    """

    VALUE_SET_NAME = 'Follow Up for Adult Depression'
    OID = '2.16.840.1.113883.3.526.3.1568'
    DEFINITION_VERSION = '20210220'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    SNOMEDCT = {
        '1555005',  # Brief group psychotherapy (regime/therapy)
        '5694008',  # Crisis intervention with follow-up (regime/therapy)
        '15558000',  # Expressive psychotherapy (regime/therapy)
        '18512000',  # Individual psychotherapy (regime/therapy)
        '28868002',  # Interactive group medical psychotherapy (regime/therapy)
        '75516001',  # Psychotherapy (regime/therapy)
        '76168009',  # Group psychotherapy (regime/therapy)
        '76740001',  # Psychiatric telephone consultation or therapy with patient (procedure)
        '81294000',  # Patient referral for psychotherapy (procedure)
        '88848003',  # Psychiatric follow-up (procedure)
        '91310009',  # Patient follow-up to return when and if necessary (procedure)
        '108313002',  # Family psychotherapy procedure (regime/therapy)
        '228557008',  # Cognitive and behavioral therapy (regime/therapy)
        '229065009',  # Exercise therapy (regime/therapy)
        '372067001',  # Implementation of measures to provide psychological support (regime/therapy)
        '385721005',  # Coping support assessment (procedure)
        '385724002',  # Coping support management (procedure)
        '385725001',  # Emotional support assessment (procedure)
        '385726000',  # Emotional support education (procedure)
        '385727009',  # Emotional support management (procedure)
        '385887004',  # Mental health history taking assessment (procedure)
        '385889001',  # Mental health history taking education (procedure)
        '385890005',  # Mental health history taking management (procedure)
        '386472008',  # Telephone consultation (procedure)
        '401277000',  # Completion of mental health crisis plan (procedure)
        '405780009',  # Dialectical behavior therapy (regime/therapy)
        '410223002',  # Mental health care assessment (procedure)
        '410224008',  # Mental health care education (procedure)
        '410225009',  # Mental health care management (procedure)
        '410226005',  # Mental health promotion assessment (procedure)
        '410227001',  # Mental health promotion education (procedure)
        '410228006',  # Mental health promotion management (procedure)
        '410229003',  # Mental health screening assessment (procedure)
        '410230008',  # Mental health screening education (procedure)
        '410231007',  # Mental health screening management (procedure)
        '410232000',  # Mental health treatment assessment (procedure)
        '410233005',  # Mental health treatment education (procedure)
        '410234004',  # Management of mental health treatment (procedure)
        '425604002',  # Case management follow up (procedure)
        '439141002',  # Discharge by mental health primary care worker (procedure)
    }


class ReferralForAdolescentDepression(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for referrals for depression management for the child and adolescent population.

    **Data Element Scope:** This value set may use a model element related to Intervention.

    **Inclusion Criteria:** Includes concepts that represent referrals for depression management in the child and adolescent population.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS2v11
    """

    VALUE_SET_NAME = 'Referral for Adolescent Depression'
    OID = '2.16.840.1.113883.3.526.3.1570'
    DEFINITION_VERSION = '20200307'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    SNOMEDCT = {
        '183524004',  # Referral to psychiatry service (procedure)
        '183583007',  # Refer to mental health worker (procedure)
        '183851006',  # Referral to clinic (procedure)
        '183866009',  # Referral to emergency clinic (procedure)
        '306136006',  # Referral to liaison psychiatry service (procedure)
        '306137002',  # Referral to mental handicap psychiatry service (procedure)
        '306226009',  # Referral to mental health counseling service (procedure)
        '306227000',  # Referral for mental health counseling (procedure)
        '306252003',  # Referral to mental health counselor (procedure)
        '306291008',  # Referral to child and adolescent psychiatrist (procedure)
        '306294000',  # Referral to psychiatrist for mental handicap (procedure)
        '308459004',  # Referral to psychologist (procedure)
        '308477009',  # Referral to psychiatrist (procedure)
        '309627007',  # Child referral - clinical psychologist (procedure)
        '390866009',  # Referral to mental health team (procedure)
        '703978000',  # Referral to primary care service (procedure)
        '710914003',  # Referral to family therapy (procedure)
        '711281004',  # Referral to support group (procedure)
    }


class ReferralForAdultDepression(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for referrals for depression management for the adult population.

    **Data Element Scope:** This value set may use a model element related to Intervention.

    **Inclusion Criteria:** Includes concepts that represent referrals for depression management in the adult population.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS2v11
    """

    VALUE_SET_NAME = 'Referral for Adult Depression'
    OID = '2.16.840.1.113883.3.526.3.1571'
    DEFINITION_VERSION = '20200307'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    SNOMEDCT = {
        '183524004',  # Referral to psychiatry service (procedure)
        '183528001',  # Referral to psychiatrist for the elderly mentally ill (procedure)
        '183583007',  # Refer to mental health worker (procedure)
        '183866009',  # Referral to emergency clinic (procedure)
        '305922005',  # Referral by mental health counselor (procedure)
        '306136006',  # Referral to liaison psychiatry service (procedure)
        '306137002',  # Referral to mental handicap psychiatry service (procedure)
        '306138007',  # Referral to psychogeriatric service (procedure)
        '306204008',  # Referral to psychogeriatric day hospital (procedure)
        '306226009',  # Referral to mental health counseling service (procedure)
        '306227000',  # Referral for mental health counseling (procedure)
        '306252003',  # Referral to mental health counselor (procedure)
        '306294000',  # Referral to psychiatrist for mental handicap (procedure)
        '308459004',  # Referral to psychologist (procedure)
        '308477009',  # Referral to psychiatrist (procedure)
        '390866009',  # Referral to mental health team (procedure)
        '703978000',  # Referral to primary care service (procedure)
        '710914003',  # Referral to family therapy (procedure)
        '711281004',  # Referral to support group (procedure)
    }


class HospiceCareAmbulatory(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of interventions to identify patients receiving hospice care outside of a hospital or long term care facility.

    **Data Element Scope:** This value set may use a model element related to Procedure or Intervention.

    **Inclusion Criteria:** Includes concepts that represent a procedure or intervention for hospice care.

    **Exclusion Criteria:** Excludes concepts that represent palliative care or comfort measures.

    ** Used in:** CMS347v5, CMS69v10
    """

    VALUE_SET_NAME = 'Hospice Care Ambulatory'
    OID = '2.16.840.1.113883.3.526.3.1584'
    DEFINITION_VERSION = '20200307'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    SNOMEDCT = {
        '385763009',  # Hospice care (regime/therapy)
        '385765002',  # Hospice care management (procedure)
    }


class Referral(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts of an encounter for a referral of a patient to a practitioner for evaluation, treatment or co-management of a patient's condition.

    **Data Element Scope:** This value set may use a model element related to Intervention.

    **Inclusion Criteria:** Includes concepts that represent an intervention for referrals and consultations.

    **Exclusion Criteria:** Excludes concepts that represent self-referrals.

    ** Used in:** CMS50v10
    """

    VALUE_SET_NAME = 'Referral'
    OID = '2.16.840.1.113883.3.464.1003.101.12.1046'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    SNOMEDCT = {
        '54395008',  # Patient referral for medical consultation (procedure)
        '103696004',  # Patient referral to specialist (procedure)
        '103697008',  # Patient referral for dental care (procedure)
        '103698003',  # Patient referral to non-physician provider (procedure)
        '103699006',  # Patient referral to dietitian (procedure)
        '103704003',  # Patient referral to sex therapist (procedure)
        '183515008',  # Referral to physician (procedure)
        '183517000',  # Referral to pediatrician (procedure)
        '183528001',  # Referral to psychiatrist for the elderly mentally ill (procedure)
        '183529009',  # Referral to oncologist (procedure)
        '183530004',  # Referral to diabetologist (procedure)
        '183541002',  # Referral to surgeon (procedure)
        '183555005',  # Burns referral (procedure)
        '183557002',  # Referral to cardiothoracic surgeon (procedure)
        '183561008',  # Referral to general practitioner (procedure)
        '183567007',  # Referral to hematologist (procedure)
        '183569005',  # Refer to terminal care consult (procedure)
        '183583007',  # Refer to mental health worker (procedure)
        '183591003',  # Refer to partner (procedure)
        '183878008',  # Private referral to general surgeon (procedure)
        '183879000',  # Private referral to ophthalmologist (procedure)
        '183880002',  # Private referral to ear, nose and throat surgeon (procedure)
        '183881003',  # Private referral to orthopedic surgeon (procedure)
        '183882005',  # Private referral to neurosurgeon (procedure)
        '183884006',  # Private referral to pediatric surgeon (procedure)
        '183885007',  # Private referral to obstetrician (procedure)
        '183886008',  # Private referral to gynecologist (procedure)
        '183887004',  # Private referral to plastic surgeon (procedure)
        '183888009',  # Private referral to oral surgeon (procedure)
        '183889001',  # Private referral to urologist (procedure)
        '183890005',  # Private referral to thoracic surgeon (procedure)
        '183891009',  # Private referral to vascular surgeon (procedure)
        '183892002',  # Private referral to maxillofacial surgeon (procedure)
        '183893007',  # Private referral cardiothoracic surgeon (procedure)
        '183894001',  # Private referral to physician (procedure)
        '183895000',  # Private referral to general physician (procedure)
        '183896004',  # Private referral to pediatrician (procedure)
        '183897008',  # Private referral to dermatologist (procedure)
        '183899006',  # Private referral to cardiologist (procedure)
        '183900001',  # Private referral to immunologist (procedure)
        '183901002',  # Private referral to neurologist (procedure)
        '183902009',  # Private referral to geriatrician (procedure)
        '183903004',  # Private referral to gastroenterologist (procedure)
        '183904005',  # Private referral to psychiatrist (procedure)
        '183905006',  # Private referral to venereologist (procedure)
        '183906007',  # Private referral to rheumatologist (procedure)
        '183907003',  # Private referral to chest physician (procedure)
        '183908008',  # Private referral to psychogeriatrician (procedure)
        '183909000',  # Private referral to oncologist (procedure)
        '183910005',  # Private referral to diabetologist (procedure)
        '183911009',  # Private referral to radiotherapist (procedure)
        '183913007',  # Private referral to geneticist (procedure)
        '183914001',  # Private referral to anesthetist (procedure)
        '183915000',  # Private referral to endocrinologist (procedure)
        '183916004',  # Private referral to nephrologist (procedure)
        '266747000',  # Referral to private doctor (procedure)
        '274410002',  # Dental referral - child (procedure)
        '306241009',  # Referral to audiological physician (procedure)
        '306242002',  # Referral to audiological scientist (procedure)
        '306243007',  # Referral to community doctor in audiology (procedure)
        '306245000',  # Referral to hearing therapist (procedure)
        '306247008',  # Referral to pediatric audiologist (procedure)
        '306250006',  # Referral to genetic counselor (procedure)
        '306252003',  # Referral to mental health counselor (procedure)
        '306253008',  # Referral to doctor (procedure)
        '306254002',  # Referral to Accident and Emergency doctor (procedure)
        '306255001',  # Referral to anesthetist (procedure)
        '306256000',  # Referral to radiotherapist (procedure)
        '306257009',  # Referral to family planning doctor (procedure)
        '306258004',  # Referral to intensive care specialist (procedure)
        '306259007',  # Referral to adult intensive care specialist (procedure)
        '306260002',  # Referral to pediatric intensive care specialist (procedure)
        '306261003',  # Referral to community pediatrician (procedure)
        '306262005',  # Referral to neonatologist (procedure)
        '306263000',  # Referral to pediatric neurologist (procedure)
        '306264006',  # Referral to pediatric oncologist (procedure)
        '306265007',  # Referral to pain management specialist (procedure)
        '306266008',  # Referral to pathologist (procedure)
        '306267004',  # Referral to blood transfusion doctor (procedure)
        '306268009',  # Referral to chemical pathologist (procedure)
        '306269001',  # Referral to general pathologist (procedure)
        '306270000',  # Referral to medical microbiologist (procedure)
        '306271001',  # Referral to neuropathologist (procedure)
        '306272008',  # Referral to clinical allergist (procedure)
        '306273003',  # Referral to chest physician (procedure)
        '306275005',  # Referral to respiratory physician (procedure)
        '306276006',  # Referral to clinical neurophysiologist (procedure)
        '306277002',  # Referral to clinical physiologist (procedure)
        '306278007',  # Referral to endocrinologist (procedure)
        '306279004',  # Referral to clinical geneticist (procedure)
        '306280001',  # Referral to clinical cytogeneticist (procedure)
        '306281002',  # Referral to clinical molecular geneticist (procedure)
        '306282009',  # Referral to genitourinary physician (procedure)
        '306284005',  # Referral to infectious diseases physician (procedure)
        '306285006',  # Referral to medical ophthalmologist (procedure)
        '306286007',  # Referral to nephrologist (procedure)
        '306287003',  # Referral to nuclear medicine physician (procedure)
        '306288008',  # Referral to palliative care physician (procedure)
        '306289000',  # Referral to rehabilitation physician (procedure)
        '306290009',  # Referral to rheumatologist (procedure)
        '306291008',  # Referral to child and adolescent psychiatrist (procedure)
        '306293006',  # Referral to liaison psychiatrist (procedure)
        '306294000',  # Referral to psychiatrist for mental handicap (procedure)
        '306295004',  # Referral to rehabilitation psychiatrist (procedure)
        '306296003',  # Referral to public health physician (procedure)
        '306297007',  # Referral to obstetrician and gynecologist (procedure)
        '306298002',  # Referral to occupational health physician (procedure)
        '306299005',  # Referral to radiologist (procedure)
        '306300002',  # Referral to breast surgeon (procedure)
        '306301003',  # Referral to thoracic surgeon (procedure)
        '306302005',  # Referral to cardiac surgeon (procedure)
        '306303000',  # Referral to dental surgeon (procedure)
        '306304006',  # Referral to orthodontist (procedure)
        '306305007',  # Referral to pediatric dentist (procedure)
        '306306008',  # Referral to restorative dentist (procedure)
        '306307004',  # Referral to endocrine surgeon (procedure)
        '306308009',  # Referral to gastrointestinal surgeon (procedure)
        '306309001',  # Referral to general gastrointestinal surgeon (procedure)
        '306310006',  # Referral to upper gastrointestinal surgeon (procedure)
        '306311005',  # Referral to colorectal surgeon (procedure)
        '306312003',  # Referral to hand surgeon (procedure)
        '306313008',  # Referral to hepatobiliary surgeon (procedure)
        '306314002',  # Referral to pancreatic surgeon (procedure)
        '306315001',  # Referral to plastic surgeon (procedure)
        '306316000',  # Referral to transplant surgeon (procedure)
        '306317009',  # Referral to trauma surgeon (procedure)
        '306318004',  # Referral to urologist (procedure)
        '306320001',  # Referral to clinical nurse specialist (procedure)
        '306338003',  # Referral to nurse practitioner (procedure)
        '306341007',  # Referral to community-based midwife (procedure)
        '306342000',  # Referral to hospital-based midwife (procedure)
        '306343005',  # Referral to psychotherapist (procedure)
        '306351008',  # Referral to community-based podiatrist (procedure)
        '306352001',  # Referral to hospital-based podiatrist (procedure)
        '306353006',  # Referral to community-based dietitian (procedure)
        '306354000',  # Referral to hospital-based dietitian (procedure)
        '306355004',  # Referral to community-based occupational therapist (procedure)
        '306356003',  # Referral to social services department occupational therapist (procedure)
        '306357007',  # Referral to hospital-based occupational therapist (procedure)
        '306358002',  # Referral to community-based physiotherapist (procedure)
        '306359005',  # Referral to hospital-based physiotherapist (procedure)
        '306360000',  # Referral to community-based speech and language therapist (procedure)
        '306361001',  # Referral to hospital-based speech and language therapist (procedure)
        '306736002',  # Referral to general dental surgeon (procedure)
        '307063001',  # Referral to clinical hematologist (procedure)
        '307777008',  # Referral to vascular surgeon (procedure)
        '308439003',  # Referral to midwife (procedure)
        '308447003',  # Referral to physiotherapist (procedure)
        '308449000',  # Referral to chiropractor (procedure)
        '308450000',  # Referral to osteopath (procedure)
        '308451001',  # Referral to podiatrist (procedure)
        '308452008',  # Referral to speech and language therapist (procedure)
        '308453003',  # Referral to occupational therapist (procedure)
        '308454009',  # Referral to orthoptist (procedure)
        '308455005',  # Referral to orthotist (procedure)
        '308456006',  # Referral to audiologist (procedure)
        '308459004',  # Referral to psychologist (procedure)
        '308465004',  # Referral to optometrist (procedure)
        '308469005',  # Referral to geneticist (procedure)
        '308470006',  # Referral to general physician (procedure)
        '308471005',  # Referral to cardiologist (procedure)
        '308472003',  # Referral to dermatologist (procedure)
        '308473008',  # Referral to clinical immunologist (procedure)
        '308474002',  # Referral to neurologist (procedure)
        '308475001',  # Referral to care of the elderly physician (procedure)
        '308476000',  # Referral to gastroenterologist (procedure)
        '308477009',  # Referral to psychiatrist (procedure)
        '308478004',  # Referral to general surgeon (procedure)
        '308479007',  # Referral to ophthalmologist (procedure)
        '308480005',  # Referral to ear, nose and throat surgeon (procedure)
        '308481009',  # Referral to orthopedic surgeon (procedure)
        '308482002',  # Referral to neurosurgeon (procedure)
        '308483007',  # Referral to pediatric surgeon (procedure)
        '308484001',  # Referral to obstetrician (procedure)
        '308485000',  # Referral to gynecologist (procedure)
        '309046007',  # Private referral to surgeon (procedure)
        '309623006',  # Child referral to optician (procedure)
        '309626003',  # Child referral - school psychologist (procedure)
        '309627007',  # Child referral - clinical psychologist (procedure)
        '309629005',  # Child referral - community dentist (procedure)
        '310515004',  # Referral to medical oncologist (procedure)
        '312487009',  # Referral to pediatric cardiologist (procedure)
        '312488004',  # Referral to community child health doctor (procedure)
        '390866009',  # Referral to mental health team (procedure)
        '401266006',  # Referral to drug abuse counselor (procedure)
        '406158007',  # Referral to oral surgeon (procedure)
        '406159004',  # Referral to maxillofacial surgeon (procedure)
        '408285001',  # Referral to pediatric dietitian (procedure)
        '415277000',  # Referral to pediatric endocrinologist (procedure)
        '416116000',  # Referral to home registered dietitian (procedure)
        '416999007',  # Private referral to physiotherapist (procedure)
        '425971006',  # Referral to pediatric pulmonologist (procedure)
        '429365000',  # Referral to pediatric gastroenterologist (procedure)
        '433151006',  # Referral to educational psychologist (procedure)
        '698563003',  # Referral to bariatric surgeon (procedure)
        '698599008',  # Private referral to breast surgeon (procedure)
        '703974003',  # Private referral to colorectal surgeon (procedure)
        '703975002',  # Private referral to podiatrist (procedure)
        '703976001',  # Private referral to radiologist (procedure)
        '716634006',  # Referral to neurological physiotherapist (procedure)
        '428441000124100',  # Referral to nurse anesthetist (procedure)
        '428451000124103',  # Referral to physician assistant (procedure)
        '428461000124101',  # Referral to nutrition professional (procedure)
        '428471000124108',  # Referral to clinical psychologist (procedure)
        '428481000124106',  # Referral to clinical social worker (procedure)
        '428491000124109',  # Referral to anesthesiologist assistant (procedure)
        '428541000124104',  # Referral to nurse midwife (procedure)
        '448761000124106',  # Referral to endovascular specialist (procedure)
        '448771000124104',  # Referral to prosthetist (procedure)
    }


class FollowUpForAboveNormalBmi(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a follow-up for a body mass index (BMI) above normal measurement.

    **Data Element Scope:** This value set may use a model element related to Intervention or Procedure.

    **Inclusion Criteria:** Includes concepts that represent a follow-up for above normal body mass index (BMI).

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS69v10
    """

    VALUE_SET_NAME = 'Follow Up for Above Normal BMI'
    OID = '2.16.840.1.113883.3.600.1.1525'
    DEFINITION_VERSION = '20190314'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    CPT = {
        '43644',  # Laparoscopy, surgical, gastric restrictive procedure; with gastric bypass and Roux-en-Y gastroenterostomy (roux limb 150 cm or less)
        '43645',  # Laparoscopy, surgical, gastric restrictive procedure; with gastric bypass and small intestine reconstruction to limit absorption
        '43659',  # Unlisted laparoscopy procedure, stomach
        '43770',  # Laparoscopy, surgical, gastric restrictive procedure; placement of adjustable gastric restrictive device (eg, gastric band and subcutaneous port components)
        '43771',  # Laparoscopy, surgical, gastric restrictive procedure; revision of adjustable gastric restrictive device component only
        '43772',  # Laparoscopy, surgical, gastric restrictive procedure; removal of adjustable gastric restrictive device component only
        '43773',  # Laparoscopy, surgical, gastric restrictive procedure; removal and replacement of adjustable gastric restrictive device component only
        '43774',  # Laparoscopy, surgical, gastric restrictive procedure; removal of adjustable gastric restrictive device and subcutaneous port components
        '43842',  # Gastric restrictive procedure, without gastric bypass, for morbid obesity; vertical-banded gastroplasty
        '43843',  # Gastric restrictive procedure, without gastric bypass, for morbid obesity; other than vertical-banded gastroplasty
        '43845',  # Gastric restrictive procedure with partial gastrectomy, pylorus-preserving duodenoileostomy and ileoileostomy (50 to 100 cm common channel) to limit absorption (biliopancreatic diversion with duodenal switch)
        '43846',  # Gastric restrictive procedure, with gastric bypass for morbid obesity; with short limb (150 cm or less) Roux-en-Y gastroenterostomy
        '43847',  # Gastric restrictive procedure, with gastric bypass for morbid obesity; with small intestine reconstruction to limit absorption
        '43848',  # Revision, open, of gastric restrictive procedure for morbid obesity, other than adjustable gastric restrictive device (separate procedure)
        '43886',  # Gastric restrictive procedure, open; revision of subcutaneous port component only
        '43888',  # Gastric restrictive procedure, open; removal and replacement of subcutaneous port component only
        '97802',  # Medical nutrition therapy; initial assessment and intervention, individual, face-to-face with the patient, each 15 minutes
        '97803',  # Medical nutrition therapy; re-assessment and intervention, individual, face-to-face with the patient, each 15 minutes
        '97804',  # Medical nutrition therapy; group (2 or more individual(s)), each 30 minutes
        '98960',  # Education and training for patient self-management by a qualified, nonphysician health care professional using a standardized curriculum, face-to-face with the patient (could include caregiver/family) each 30 minutes; individual patient
        '99078',  # Physician or other qualified health care professional qualified by education, training, licensure/regulation (when applicable) educational services rendered to patients in a group setting (eg, prenatal, obesity, or diabetic instructions)
        '99401',  # Preventive medicine counseling and/or risk factor reduction intervention(s) provided to an individual (separate procedure); approximately 15 minutes
        '99402',  # Preventive medicine counseling and/or risk factor reduction intervention(s) provided to an individual (separate procedure); approximately 30 minutes
    }
    HCPCSLEVELII = {
        'G0270',  # Medical nutrition therapy; reassessment and subsequent intervention(s) following second referral in same year for change in diagnosis, medical condition or treatment regimen (including additional hours needed for renal disease), individual, face to face with the patient, each 15 minutes
        'G0271',  # Medical nutrition therapy, reassessment and subsequent intervention(s) following second referral in same year for change in diagnosis, medical condition, or treatment regimen (including additional hours needed for renal disease), group (2 or more individuals), each 30 minutes
        'G0447',  # Face-to-face behavioral counseling for obesity, 15 minutes
        'G0473',  # Face-to-face behavioral counseling for obesity, group (2-10), 30 minutes
        'S9449',  # Weight management classes, non-physician provider, per session
        'S9451',  # Exercise classes, non-physician provider, per session
        'S9452',  # Nutrition classes, non-physician provider, per session
        'S9470',  # Nutritional counseling, dietitian visit
    }
    ICD10CM = {
        'Z71.3',  # Dietary counseling and surveillance
        'Z71.82',  # Exercise counseling
    }
    SNOMEDCT = {
        '304549008',  # Giving encouragement to exercise (procedure)
        '307818003',  # Weight monitoring (regime/therapy)
        '361231003',  # Prescribed dietary intake (regime/therapy)
        '370847001',  # Dietary needs education (procedure)
        '386291006',  # Exercise promotion: strength training (procedure)
        '386292004',  # Exercise promotion: stretching (procedure)
        '386373004',  # Nutrition therapy (regime/therapy)
        '386463000',  # Prescribed activity/exercise education (procedure)
        '386464006',  # Prescribed diet education (procedure)
        '410177006',  # Special diet education (procedure)
        '413315001',  # Nutrition / feeding management (regime/therapy)
        '418995006',  # Feeding regime (regime/therapy)
        '424753004',  # Dietary management education, guidance, and counseling (procedure)
        '443288003',  # Lifestyle education regarding diet (procedure)
    }


class FollowUpForBelowNormalBmi(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for follow-up with a body mass index (BMI) below normal measurement.

    **Data Element Scope:** This value set may use a model element related to Intervention or Procedure.

    **Inclusion Criteria:** Includes concepts that represent a follow-up for below normal body mass index (BMI).

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS69v10
    """

    VALUE_SET_NAME = 'Follow Up for Below Normal BMI'
    OID = '2.16.840.1.113883.3.600.1.1528'
    DEFINITION_VERSION = '20200307'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    CPT = {
        '97802',  # Medical nutrition therapy; initial assessment and intervention, individual, face-to-face with the patient, each 15 minutes
        '97803',  # Medical nutrition therapy; re-assessment and intervention, individual, face-to-face with the patient, each 15 minutes
        '97804',  # Medical nutrition therapy; group (2 or more individual(s)), each 30 minutes
        '98960',  # Education and training for patient self-management by a qualified, nonphysician health care professional using a standardized curriculum, face-to-face with the patient (could include caregiver/family) each 30 minutes; individual patient
        '99078',  # Physician or other qualified health care professional qualified by education, training, licensure/regulation (when applicable) educational services rendered to patients in a group setting (eg, prenatal, obesity, or diabetic instructions)
        '99401',  # Preventive medicine counseling and/or risk factor reduction intervention(s) provided to an individual (separate procedure); approximately 15 minutes
        '99402',  # Preventive medicine counseling and/or risk factor reduction intervention(s) provided to an individual (separate procedure); approximately 30 minutes
    }
    HCPCSLEVELII = {
        'G0270',  # Medical nutrition therapy; reassessment and subsequent intervention(s) following second referral in same year for change in diagnosis, medical condition or treatment regimen (including additional hours needed for renal disease), individual, face to face with the patient, each 15 minutes
        'G0271',  # Medical nutrition therapy, reassessment and subsequent intervention(s) following second referral in same year for change in diagnosis, medical condition, or treatment regimen (including additional hours needed for renal disease), group (2 or more individuals), each 30 minutes
        'S9449',  # Weight management classes, non-physician provider, per session
        'S9452',  # Nutrition classes, non-physician provider, per session
        'S9470',  # Nutritional counseling, dietitian visit
    }
    ICD10CM = {
        'Z71.3',  # Dietary counseling and surveillance
    }
    SNOMEDCT = {
        '386464006',  # Prescribed diet education (procedure)
        '410177006',  # Special diet education (procedure)
        '413315001',  # Nutrition / feeding management (regime/therapy)
        '418995006',  # Feeding regime (regime/therapy)
        '424753004',  # Dietary management education, guidance, and counseling (procedure)
        '429095004',  # Dietary education for weight gain (procedure)
        '443288003',  # Lifestyle education regarding diet (procedure)
    }


class ReferralsWhereWeightAssessmentMayOccur(ValueSet):
    """
    **Clinical Focus:** The purpose of this value set is to represent concepts for a referral to multiple types of providers and settings for weight assessment.

    **Data Element Scope:** This value set may use a model element related to Intervention or Procedure.

    **Inclusion Criteria:** Includes concepts that represent a referral to multiple types of providers and settings for weight assessment.

    **Exclusion Criteria:** No exclusions.

    ** Used in:** CMS69v10
    """

    VALUE_SET_NAME = 'Referrals Where Weight Assessment May Occur'
    OID = '2.16.840.1.113883.3.600.1.1527'
    DEFINITION_VERSION = '20170504'
    EXPANSION_VERSION = 'eCQM Update 2021-05-06'

    SNOMEDCT = {
        '183515008',  # Referral to physician (procedure)
        '183524004',  # Referral to psychiatry service (procedure)
        '183583007',  # Refer to mental health worker (procedure)
        '306136006',  # Referral to liaison psychiatry service (procedure)
        '306163007',  # Referral to dietetics service (procedure)
        '306164001',  # Referral to community-based dietetics service (procedure)
        '306165000',  # Referral to hospital-based dietetics service (procedure)
        '306166004',  # Referral to occupational therapy service (procedure)
        '306167008',  # Referral to community-based occupational therapy service (procedure)
        '306168003',  # Referral to hospital-based occupational therapy service (procedure)
        '306226009',  # Referral to mental health counseling service (procedure)
        '306227000',  # Referral for mental health counseling (procedure)
        '306252003',  # Referral to mental health counselor (procedure)
        '306344004',  # Referral to professional allied to medicine (procedure)
        '306353006',  # Referral to community-based dietitian (procedure)
        '306354000',  # Referral to hospital-based dietitian (procedure)
        '308459004',  # Referral to psychologist (procedure)
        '308470006',  # Referral to general physician (procedure)
        '308477009',  # Referral to psychiatrist (procedure)
        '390864007',  # Referral for exercise therapy (procedure)
        '390866009',  # Referral to mental health team (procedure)
        '390893007',  # Referral to physical activity program (procedure)
        '408289007',  # Refer to weight management program (procedure)
        '416790000',  # Referral for home physical therapy (procedure)
    }




from canvas_workflow_kit.recommendation import Recommendation, LabRecommendation
from canvas_workflow_kit.value_set.v2018 import Diabetes, Hba1CLaboratoryTest

from .base import SDKBaseTest


class SDKLabTest(SDKBaseTest):

    def test_lab_recommendation(self):
        patient = self.load_patient('example')
        self.assertTrue(patient)

        rec = LabRecommendation(
            key='KEY-ID',
            rank=123,
            button='ACT',
            patient=patient,
            lab=Hba1CLaboratoryTest,
            condition=Diabetes)
        self.assertTrue(rec)
        self.assertIn('Diabetes', rec.narrative)
        self.assertIn('HbA1c', rec.narrative)
        self.assertIn('lab', rec.command['type'])

    def test_basic_recommendation(self):
        rec = Recommendation(
            key='KEY-ID',
            rank=123,
            button='ACT',
            title='Do Something',
            narrative="'I'm not sure what, but do something!'")
        self.assertTrue(rec)
        self.assertEqual(rec.command, {})



from protocols import events
from protocols.protocol import ClinicalQualityMeasure
from protocols.value_set.v2021 import (
    EncounterInpatient, Ethnicity, FractureLowerBody,
    HipDysfunctionAndOsteoarthritisOutcomeScoreForJointReplacementHoosjr, HospiceCareAmbulatory,
    OfficeVisit, OncAdministrativeSex, OutpatientConsultation, Payer, Race)


class ClinicalQualityMeasure56v9(ClinicalQualityMeasure):
    """
    Functional Status Assessment for Total Hip Replacement

    Description: Percentage of patients 18 years of age and older who received an elective primary
    total hip arthroplasty (THA) and completed a functional status assessment within 90 days prior
    to the surgery and in the 270-365 days after the surgery

    Definition: None

    Rationale: Total hip arthroplasties (THAs) are common surgical procedures that address hip pain
    and functional impairment, primarily caused by osteoarthritis. Although THA is an effective
    procedure for addressing osteoarthritis for many patients, some people, particularly those with
    more severe preoperative pain and impairment, do not experience the improvements in pain,
    function, and quality of life expected from the procedure (Beswick et al., 2012; Fortin et al.,
    1999; Tilbury et al., 2016). In 2010, providers performed 326,100 THAs, with 95 percent of them
    in patients age 45 and older (Wolford, Palso, & Bercovitz, 2015). Although THAs were introduced
    as a procedure for older adults, the percentage of patients age 55 to 64 (29 percent) who had a
    THA in 2010 exceeded the percentage of patients age 75 and older (26 percent) who had a THA
    (Wolford, Palso, & Bercovitz, 2015). Kurtz et al. (2009) projected that patients younger than
    65 would account for 52 percent of THAs by 2030. This growth in hip surgeries for patients
    younger than 65 is significant because these patients often require more expensive joint
    arthroplasties that will better withstand the wear caused by physical activity (Bozic et al.,
    2006).

    This measure evaluates whether patients complete a patient-reported functional status
    assessment (FSA) before and after a THA. Measuring functional status for patients undergoing
    THA permits longitudinal assessment - from the patient's perspective - of the impact of
    surgical intervention on pain, physical function, as well as health-related quality of life
    (Rothrock, 2010).

    Guidance: The same functional status assessment (FSA) instrument must be used for the initial
    and follow-up assessment.

    This eCQM is a patient-based measure.

    This version of the eCQM uses QDM version 5.5.  Please refer to the eCQI resource center
    (https://ecqi.healthit.gov/qdm) for more information on the QDM.

    More information: https://ecqi.healthit.gov/ecqm/measures/cms56v9
    """

    title = 'Functional Status Assessment for Total Hip Replacement'

    identifiers = ['CMS56v9']

    types = ['CQM']

    responds_to_event_types = [
        events.HEALTH_MAINTENANCE,
    ]

    authors = [
        'National Committee for Quality Assurance',
    ]

    references = [
        'American Association of Orthopaedic Surgeons, the American Joint Replacement Registry, The Hip Society, The Knee Society, & the American Association of Hip and Knee Surgeons. (2015). Patient-reported outcomes summit for total joint arthroplasty report. Retrieved from http://www.aahks.org/wp-content/uploads/2018/08/comment-letter-pro-summit-09082015.pdf',
        'Beswick, A. D., Wylde, V., Gooberman-Hill, R., et al. (2012). What proportion of patients report long-term pain after total hip or knee replacement for osteoarthritis? A systematic review of prospective studies in unselected patients. British Medical Journal Open, 2(1), 1-12.',
        'Bozic, K. J., Morsehed, S., Silverstein, M. D., et al. (2006). Use of cost-effectiveness analysis to evaluate new technologies in orthopaedics: The case of alternative bearing surfaces in total hip arthroplasty. Journal of Bone and Joint Surgery, American Volume, 88(4), 706-714.',
        'Fortin, P. R., Clarke, A. E., Joseph, L., et al. (1999). Outcomes of total hip and knee replacement preoperative functional status predicts outcomes at six months after surgery. American College of Rheumatology, 42(8), 1722-1728.',
        'Kurtz, S. M., Lau, E., Ong, K., et al. (2009). Future young patient demand for primary and revision joint arthroplasty: National projections from 2010 to 2030. Clinical Orthopaedics and Related Research, 467(10), 2606-2612.',
        'Rothrock, N. E., Hays, R. D., Spritzer, K., et al. (2010). Relative to the general U.S. population, chronic diseases are associated with poorer health-related quality of life as measured by the Patient-Reported Outcomes Measurement Information System (PROMIS). Journal of Clinical Epidemiology, 63(11), 1195-1204.',
        'Tilbury, C., Haanstra, T. M., Leichtenberg, C. S., et al. (2016). Unfulfilled expectations after total hip and knee arthroplasty surgery: There is a need for better preoperative patient information and education. Journal of Arthroplasty, 31(10), 2136-2145.',
        'Wolford, M. L., Palso, K., & Bercovitz, A. (2015, February). Hospitalization for total hip replacement among inpatients aged 45 and over: United States, 2000-2010. NCHS Data Brief No. 186. Hyattsville, MD: National Center for Health Statistics.',
    ]

    funding_source = ''

    def in_initial_population(self):
        """
        Initial population: Patients 19 years of age and older who had a primary total hip
        arthroplasty (THA) in the year prior to the measurement period and who had an outpatient
        encounter during the measurement period
        """
        pass

    def in_denominator(self):
        """
        Denominator: Equals Initial Population

        Exclusions: Patients with two or more fractures indicating trauma at the time of the total
        hip arthroplasty or patients with severe cognitive impairment that overlaps the measurement
        period.

        Exclude patients whose hospice care overlaps the measurement period.

        Exceptions: None
        """
        return self.in_initial_population()

    def in_numerator(self):
        """
        Numerator: Patients with patient-reported functional status assessment  results (i.e.,
        Veterans RAND 12-item health survey [VR-12], Patient-Reported Outcomes Measurement
        Information System [PROMIS]-10-Global Health, Hip Disability and Osteoarthritis Outcome
        Score [HOOS], HOOS Jr.) in the 90 days prior to or on the day of the primary THA procedure,
        and in the 270 - 365 days after the THA procedure

        Exclusions: Not Applicable
        """
        pass


    def compute_results(self):
        """
        Clinical recommendation: While there is no clinical guideline recommending that clinicians
        assess patients who are undergoing total hip replacements using patient-reported outcomes
        of function and pain, several clinical specialty societies support the use of a general
        health questionnaire and a disease-specific questionnaire for these patients. In
        particular, they recommend the Veterans RAND 12-item health survey (VR-12) or the Patient-
        Reported Outcomes Measurement Information System [PROMIS]-10-Global as the general health
        questionnaire and the Hip Disability and Osteoarthritis Outcome Score [HOOS], Jr. as the
        disease-specific questionnaire (American Association of Orthopaedic Surgeons, the American
        Joint Replacement Registry, The Hip Society, The Knee Society, & the American Association
        of Hip and Knee Surgeons, 2015).
        """
        pass



from typing import List, Optional, cast

import arrow

from canvas_workflow_kit import events
from canvas_workflow_kit.protocol import (
    CONTEXT_REPORT,
    STATUS_DUE,
    STATUS_SATISFIED,
    ClinicalQualityMeasure,
    ProtocolResult
)
from canvas_workflow_kit.recommendation import InstructionRecommendation
# @canvas-adr-0006
from canvas_workflow_kit.value_set.specials import CMS125v6Tomography
# flake8: noqa
from canvas_workflow_kit.value_set.v2018 import (
    AnnualWellnessVisit,
    BilateralMastectomy,
    DischargedToHealthCareFacilityForHospiceCare,
    DischargedToHomeForHospiceCare,
    EncounterInpatient,
    Ethnicity,
    FaceToFaceInteraction,
    Female,
    HistoryOfBilateralMastectomy,
    HomeHealthcareServices,
    HospiceCareAmbulatory,
    Mammography,
    OfficeVisit,
    OncAdministrativeSex,
    Payer,
    PreventiveCareServicesEstablishedOfficeVisit18AndUp,
    PreventiveCareServicesInitialOfficeVisit18AndUp,
    Race,
    StatusPostLeftMastectomy,
    StatusPostRightMastectomy,
    UnilateralMastectomy,
    UnilateralMastectomyUnspecifiedLaterality
)


class ClinicalQualityMeasure125v6(ClinicalQualityMeasure):
    """
    Breast Cancer Screening

    Description: Percentage of women 50-74 years of age who had a mammogram to screen for breast
    cancer

    Definition: None

    Rationale: Breast cancer is one of the most common types of cancers, accounting for a quarter
    of all new cancer diagnoses for women in the U.S. (BreastCancer.Org, 2011). It ranks as the
    second leading cause of cancer-related mortality in women, accounting for nearly 40,000
    estimated deaths in 2013 (American Cancer Society, 2011).

    According to the National Cancer Institute's Surveillance Epidemiology and End Results program,
    the chance of a woman being diagnosed with breast cancer in a given year increases with age. By
    age 30, it is one in 2,212. By age 40, the chances increase to one in 235, by age 50, it
    becomes one in 54, and, by age 60, it is one in 25. From 2004 to 2008, the median age at the
    time of breast cancer diagnosis was 61 years among adult women (Tangka et al, 2010).

    In the U.S., costs associated with a diagnosis of breast cancer range from $451 to $2,520,
    factoring in continued testing, multiple office visits, and varying procedures. The total costs
    related to breast cancer add up to nearly $7 billion per year in the U.S., including $2 billion
    spent on late-stage treatment (Lavigne et al, 2008; Boykoff et al, 2009).

    Guidance: Patient self-report for procedures as well as diagnostic studies should be recorded
    in 'Procedure, Performed' template or 'Diagnostic Study, Performed' template in QRDA-1. Patient
    self-report is not allowed for laboratory tests.

    This measure evaluates primary screening. Do not count biopsies, breast ultrasounds, MRIs or
    tomosynthesis (3D mammography), because they are not appropriate methods for primary breast
    cancer screening.

    More information: https://ecqi.healthit.gov/sites/default/files/ecqm/measures/CMS125v6.html
    """

    class Meta:
        title = 'Breast Cancer Screening'
        version = '2002-02-12v1'

        # 27 months, but we want to ensure it is displayed as 2 years, 3 months
        # 27 * 30 is 2 years, 2 months, and some amount of days
        default_display_interval_in_days = (365 * 2) + (3 * 30)

        description = ('Women 50-74 years of age who have not had a mammogram to screen for '
                       'breast cancer within the last 27 months.')
        information = 'https://ecqi.healthit.gov/sites/default/files/ecqm/measures/CMS125v6.html'

        identifiers = ['CMS125v6']

        types = ['CQM']

        responds_to_event_types = [
            events.HEALTH_MAINTENANCE,
        ]

        authors = [
            'National Committee for Quality Assurance',
        ]

        references = [
            'American Cancer Society. 2010. Cancer Facts & Figures 2010. Atlanta: American Cancer Society.',
            'National Cancer Institute. 2010. "Breast Cancer Screening." http://www.cancer.gov/cancertopics/pdq/screening/breast/healthprofessional',
            'National Business Group on Health. 2011. Pathways to Managing Cancer in the Workplace. Washington: National Business Group on Health.',
            'U.S. Preventive Services Task Force (USPSTF). 2009. 1) "Screening for breast cancer: U.S. Preventive Services Task Force recommendation statement." 2) "December 2009 addendum." Ann Intern Med 151(10):716-726.',
            'BreastCancer.org. 2012. U.S. Breast Cancer Statistics. www.breastcancer.org/symptoms/understand_bc/statistics.jsp', ]

        compute_on_change_types = [
            ClinicalQualityMeasure.CHANGE_PROTOCOL_OVERRIDE,
            ClinicalQualityMeasure.CHANGE_CONDITION,
            ClinicalQualityMeasure.CHANGE_IMAGING_REPORT,
            ClinicalQualityMeasure.CHANGE_PATIENT,
        ]

    _on_date = None
    AGE_RANGE_START = 51
    AGE_RANGE_END = 74
    EXTRA_SCREENING_MONTHS = 15

    def had_mastectomy(self) -> bool:
        if self.patient.conditions.find(BilateralMastectomy).starts_before(self.timeframe.end):
            return True

        unilateral_mastectomy = self.patient.conditions.find(UnilateralMastectomy).starts_before(
            self.timeframe.end)
        if 2 == len(unilateral_mastectomy):
            return True

        if unilateral_mastectomy and self.patient.conditions.find(
                StatusPostRightMastectomy).starts_before(self.timeframe.end):
            return True

        if unilateral_mastectomy and self.patient.conditions.find(
                StatusPostLeftMastectomy).starts_before(self.timeframe.end):
            return True

        return False

    def first_due_in(self) -> Optional[int]:
        if (
                self.patient.is_female and
                self.patient.age_at(self.timeframe.end) < self.AGE_RANGE_START and
                not self.had_mastectomy()):  # yapf: disable
            return cast(int,
                        (self.patient.birthday.shift(years=self.AGE_RANGE_START) -
                         self.timeframe.end).days)
        return None

    def in_initial_population(self) -> bool:
        """
        Initial population: Women 51-74 years of age with a visit during the measurement period
        """
        return cast(bool, self.patient.age_at_between(self.timeframe.end,
                                                      self.AGE_RANGE_START,
                                                      self.AGE_RANGE_END) and
                    self.patient.is_female and
                    (self.patient.has_visit_within(self.timeframe,
                                                   (OfficeVisit |
                                                    PreventiveCareServicesInitialOfficeVisit18AndUp |
                                                    PreventiveCareServicesEstablishedOfficeVisit18AndUp |
                                                    HomeHealthcareServices |
                                                    AnnualWellnessVisit))
                     if self.context == CONTEXT_REPORT else True))  # yapf: disable

    def in_denominator(self) -> bool:
        """
        Denominator: Equals Initial Population

        Exclusions: Women who had a bilateral mastectomy or who have a history of a bilateral
        mastectomy or for whom there is evidence of a right and a left unilateral mastectomy.

        Exclude patients who were in hospice care during the measurement year.

        Exceptions: None
        """
        if not self.in_initial_population():
            return False

        if self.patient.hospice_within(self.timeframe):
            return False

        if self.had_mastectomy():
            return False

        return True

    def in_numerator(self) -> bool:
        """
        Numerator: Women with one or more mammograms during the measurement period or the 15 months
        prior to the measurement period

        Exclusions: Not Applicable
        """
        if self.period_adjustment:
            period = self.timeframe
        else:
            period = self.timeframe.increased_by(months=-1 * self.EXTRA_SCREENING_MONTHS)
        record = self.patient.imaging_reports.find(Mammography |
                                                   CMS125v6Tomography).within(period).last()
        if record:
            self._on_date = arrow.get(record['originalDate'])
            return True

        return False

    def compute_results(self) -> ProtocolResult:
        """
        Clinical recommendation: The U.S. Preventive Services Task Force (USPSTF) recommends
        biennial screening mammography for women aged 50-74 years (B recommendation). The decision
        to start regular, biennial screening mammography before the age of 50 years should be an
        individual one and take patient context into account, including the patient's values
        regarding specific benefits and harms (C recommendation). The Task Force concludes the
        evidence is insufficient to assess the additional benefits and harms of screening
        mammography in women 75 years and older (I statement).

        U.S. Preventive Services Task Force (2009)
        Grade: B recommendation. The USPSTF recommends biennial screening mammography for women
        aged 50 to 74 years.
        Grade: C recommendation. The decision to start regular, biennial screening mammography
        before the age of 50 years should be an individual one and take patient context into
        account, including the patient's values regarding specific benefits and harms.
        Grade: I Statement. The USPSTF concludes that the current evidence is insufficient to
        assess the additional benefits and harms of screening mammography in women 75 years or
        older.
        Grade: D recommendation. The USPSTF recommends against teaching breast self-examination
        (BSE).
        Grade: I Statement. The USPSTF concludes that the current evidence is insufficient to
        assess the additional benefits and harms of clinical breast examination (CBE) beyond
        screening mammography in women 40 years or older.
        Grade: I Statement. The USPSTF concludes that the current evidence is insufficient to
        assess the additional benefits and harms of either digital mammography or magnetic
        resonance imaging (MRI) instead of film mammography as screening modalities for breast
        cancer.
        """
        result = ProtocolResult()

        if self.in_denominator():
            first_name = self.patient.first_name
            extra_months = 0 if self.period_adjustment else self.EXTRA_SCREENING_MONTHS
            if self.in_numerator() and self._on_date:

                result.due_in = (self._on_date.shift(
                    days=self.timeframe.duration, months=extra_months) - self.now).days
                result.status = STATUS_SATISFIED
                result.add_narrative(
                    f'{first_name} had a mammography {self.display_date(self._on_date)}.')
            else:
                result.due_in = -1
                result.status = STATUS_DUE
                result.add_narrative('No relevant exams found.')  # yapf: disable
                result.add_narrative(self.screening_interval_context())
                title = 'Discuss breast cancer screening and order imaging as appropriate'
                result.add_recommendation(
                    InstructionRecommendation(
                        key='CMS125v6_RECOMMEND_MAMMOGRAPHY',
                        rank=1,
                        button='Plan',
                        patient=self.patient,
                        instruction=Mammography,
                        title=title))
        else:
            result.due_in = self.first_due_in()

        return result



import io

from contextlib import redirect_stdout
from datetime import datetime, timedelta

import arrow

from canvas_workflow_kit.patient_recordset import BillingLineItemRecordSet, PatientRecordSet
from canvas_workflow_kit.tests.base import SDKBaseTest
from canvas_workflow_kit.timeframe import Timeframe
from canvas_workflow_kit.value_set.v2018 import AnnualWellnessVisit, HomeHealthcareServices, OfficeVisit


class TestPatient(SDKBaseTest):

    def setUp(self):
        super().setUp()

    def test___init__(self):
        # no data
        tested = self.load_patient('empty')
        self.assertFalse(tested.active_only)
        self.assertEqual('empty', tested.patient_key)
        fields = [
            'billing_line_items',
            'conditions',
            'imaging_reports',
            'immunizations',
            'inpatient_stays',
            'instructions',
            'interviews',
            'lab_reports',
            'medications',
            'procedures',
            'protocol_overrides',
            'prescriptions',
            'referral_reports',
            'vital_signs',
            'suspect_hccs',
        ]
        for field in fields:
            self.assertTrue(hasattr(tested, field), 'patient misses the field %s' % field)
        # some data
        tested = self.load_patient('partial')
        self.assertFalse(tested.active_only)
        self.assertEqual('partial', tested.patient_key)

        fields = {
            'billing_line_items': 0,
            'conditions': 2,
            'imaging_reports': 1,
            'immunizations': 0,
            'inpatient_stays': 0,
            'instructions': 0,
            'interviews': 1,
            'lab_reports': 1,
            'medications': 1,
            'procedures': 0,
            'protocol_overrides': 0,
            'prescriptions': 0,
            'referral_reports': 0,
            'vital_signs': 2,
            'suspect_hccs': 0,
        }
        for field in fields:
            self.assertTrue(hasattr(tested, field), 'patient misses the field %s' % field)
            records = getattr(tested, field)
            self.assertEqual(fields[field], len(records),
                             '`%s` expected records %d' % (field, fields[field]))

        # # all data
        tested = self.load_patient('full')
        self.assertFalse(tested.active_only)
        self.assertEqual('full', tested.patient_key)
        fields = {
            'billing_line_items': 2,
            'conditions': 3,
            'imaging_reports': 2,
            'immunizations': 1,
            'inpatient_stays': 0,
            'instructions': 1,
            'interviews': 2,
            'lab_reports': 2,
            'medications': 2,
            'procedures': 1,
            'protocol_overrides': 2,
            'prescriptions': 1,
            'referral_reports': 1,
            'vital_signs': 3,
            'suspect_hccs': 0,
        }
        for field in fields:
            self.assertTrue(hasattr(tested, field), 'patient misses the field %s' % field)
            records = getattr(tested, field)
            self.assertEqual(fields[field], len(records),
                             '`%s` expected records %d' % (field, fields[field]))

    def test_recordset_classes(self):
        tested = self.load_patient('full')
        expected = [
            'AdministrativeDocumentRecordSet',
            'AllergyIntoleranceRecordSet',
            'BillingLineItemRecordSet',
            'ConditionRecordSet',
            'ImagingReportRecordSet',
            'ImmunizationRecordSet',
            'InpatientStayRecordSet',
            'InstructionRecordSet',
            'InterviewRecordSet',
            'LabReportRecordSet',
            'MedicationRecordSet',
            'ProcedureRecordSet',
            'ProtocolOverrideRecordSet',
            'PrescriptionRecordSet',
            'ReasonForVisitRecordSet',
            'ReferralRecordSet',
            'ReferralReportRecordSet',
            'UpcomingAppointmentNoteRecordSet',
            'UpcomingAppointmentRecordSet',
            'VitalSignRecordSet',
        ]
        count = 0
        for cls in tested.RECORDSET_CLASSES:
            self.assertTrue(issubclass(cls, PatientRecordSet), 'class %s' % cls.__name__)
            self.assertTrue(cls.__name__ in expected, 'class %s' % cls.__name__)
            count += 1
        self.assertEqual(len(expected), count)

    def test_recordset_fields(self):
        tested = self.load_patient('full')
        expected = [
            'administrative_documents',
            'allergy_intolerances',
            'billing_line_items',
            'conditions',
            'imaging_reports',
            'immunizations',
            'inpatient_stays',
            'instructions',
            'interviews',
            'lab_reports',
            'medications',
            'procedures',
            'protocol_overrides',
            'prescriptions',
            'reason_for_visits',
            'referrals',
            'referral_reports',
            'upcoming_appointment_notes',
            'upcoming_appointments',
            'vital_signs',
        ]
        self.assertEqual(expected, tested.recordset_fields())

    def test_first_name(self):
        tested = self.load_patient('empty')
        self.assertEqual('Jojo', tested.first_name)
        tested = self.load_patient('full')
        self.assertEqual('Darey', tested.first_name)
        tested = self.load_patient('partial')
        self.assertEqual('Toure', tested.first_name)

    def test_is_female(self):
        tested = self.load_patient('empty')
        self.assertFalse(tested.is_female)
        tested = self.load_patient('full')
        self.assertTrue(tested.is_female)
        tested = self.load_patient('partial')
        self.assertFalse(tested.is_female)

    def test_is_male(self):
        tested = self.load_patient('empty')
        self.assertFalse(tested.is_male)
        tested = self.load_patient('full')
        self.assertFalse(tested.is_male)
        tested = self.load_patient('partial')
        self.assertTrue(tested.is_male)

    def test_is_african_american(self):
        tested = self.load_patient('empty')
        self.assertFalse(tested.is_african_american)
        tested = self.load_patient('full')
        self.assertTrue(tested.is_african_american)
        tested = self.load_patient('partial')
        self.assertFalse(tested.is_african_american)

    def test_birthday(self):
        tested = self.load_patient('full')  # 1959-06-11
        self.assertIsInstance(tested.birthday, arrow.Arrow)
        self.assertEqual('1959-06-11', tested.birthday.format('YYYY-MM-DD'))

    def test_age(self):
        tested = self.load_patient('empty')
        expected = self.helper_age(1997, 7, 26)
        self.assertEqual(expected, tested.age)

        tested = self.load_patient('full')
        expected = self.helper_age(1959, 6, 11)
        self.assertEqual(expected, tested.age)

        tested = self.load_patient('partial')
        expected = self.helper_age(1974, 3, 24)
        self.assertEqual(expected, tested.age)

    def test_age_at(self):
        tested = self.load_patient('full')  # 1959-06-11

        self.assertAlmostEqual(
            49.997, tested.age_at(arrow.get('2009-06-10', 'YYYY-MM-DD')), places=3)
        self.assertAlmostEqual(
            50.000, tested.age_at(arrow.get('2009-06-11', 'YYYY-MM-DD')), places=3)
        self.assertAlmostEqual(
            50.003, tested.age_at(arrow.get('2009-06-12', 'YYYY-MM-DD')), places=3)
        self.assertAlmostEqual(
            50.501, tested.age_at(arrow.get('2009-12-11', 'YYYY-MM-DD')), places=3)

        # on leap year for date preceding the birthday and after the 02/29, the fraction is higher
        self.assertAlmostEqual(
            48.7213, tested.age_at(arrow.get('2008-03-01', 'YYYY-MM-DD')), places=4)
        self.assertAlmostEqual(
            49.7205, tested.age_at(arrow.get('2009-03-01', 'YYYY-MM-DD')), places=4)
        self.assertAlmostEqual(
            50.7205, tested.age_at(arrow.get('2010-03-01', 'YYYY-MM-DD')), places=4)
        self.assertAlmostEqual(
            51.7205, tested.age_at(arrow.get('2011-03-01', 'YYYY-MM-DD')), places=4)

        # on leap year for date following the birthday and befpre the 02/29, the fraction is lower
        self.assertAlmostEqual(
            49.2247, tested.age_at(arrow.get('2008-09-01', 'YYYY-MM-DD')), places=4)
        self.assertAlmostEqual(
            50.2247, tested.age_at(arrow.get('2009-09-01', 'YYYY-MM-DD')), places=4)
        self.assertAlmostEqual(
            51.2247, tested.age_at(arrow.get('2010-09-01', 'YYYY-MM-DD')), places=4)
        self.assertAlmostEqual(
            52.2240, tested.age_at(arrow.get('2011-09-01', 'YYYY-MM-DD')), places=4)

        # patient does NOT exist
        tested.patient = []
        self.assertEqual(0, tested.age_at(arrow.get('2011-09-01', 'YYYY-MM-DD')))

    def test_age_at_between(self):
        tested = self.load_patient('full')  # 1959-06-11
        self.assertTrue(tested.age_at_between(arrow.get('2009-06-10', 'YYYY-MM-DD'), 7, 50))
        self.assertFalse(tested.age_at_between(arrow.get('2009-06-10', 'YYYY-MM-DD'), 7, 49))

        self.assertTrue(tested.age_at_between(arrow.get('2009-06-11', 'YYYY-MM-DD'), 7, 51))
        self.assertFalse(tested.age_at_between(arrow.get('2009-06-11', 'YYYY-MM-DD'), 7, 50))

        self.assertTrue(tested.age_at_between(arrow.get('2009-06-12', 'YYYY-MM-DD'), 7, 51))
        self.assertFalse(tested.age_at_between(arrow.get('2009-06-12', 'YYYY-MM-DD'), 7, 50))

    def test_hospice_within(self):
        tested = self.load_patient('full')
        start = arrow.get('1999-06-10', 'YYYY-MM-DD')
        end = arrow.get('2109-06-10', 'YYYY-MM-DD')
        timeframe = Timeframe(start=start, end=end)
        self.assertFalse(tested.hospice_within(timeframe))

    def test_has_visit_within(self):
        tested = self.load_patient('full')
        start = arrow.get('2018-03-05', 'YYYY-MM-DD')
        end = arrow.get('2109-03-05', 'YYYY-MM-DD')
        timeframe = Timeframe(start=start, end=end)

        # no ValueSet provided --> True
        result = tested.has_visit_within(timeframe, None)
        self.assertTrue(result)

        tested.billing_line_items = BillingLineItemRecordSet([{
            'id': 92,
            'created': '2018-08-05T21:41:21.407046Z',
            'datetimeOfService': '2018-08-05T21:41:21.407046Z',
            'cpt': '99202',
            'units': 1
        }])
        # incorrect ValueSet + billing item --> False
        result = tested.has_visit_within(timeframe, AnnualWellnessVisit)
        self.assertFalse(result)
        # correct ValueSet + billing item + date within the time frame --> False
        result = tested.has_visit_within(timeframe, OfficeVisit)
        self.assertTrue(result)
        result = tested.has_visit_within(
            timeframe, AnnualWellnessVisit | OfficeVisit | HomeHealthcareServices)
        self.assertTrue(result)

        # date outside the time frame
        tested.billing_line_items = BillingLineItemRecordSet([{
            'id': 92,
            'created': '2018-02-05T21:41:21.407046Z',
            'datetimeOfService': '2018-02-05T21:41:21.407046Z',
            'cpt': '99202',
            'units': 1
        }])
        result = tested.has_visit_within(timeframe, OfficeVisit)
        self.assertFalse(result)

    def test_count_visit_within(self):
        tested = self.load_patient('full')
        start = arrow.get('2018-03-05', 'YYYY-MM-DD')
        end = arrow.get('2019-03-05', 'YYYY-MM-DD')
        timeframe = Timeframe(start=start, end=end)

        # no ValueSet provided --> True
        result = tested.count_visit_within(timeframe, None)
        self.assertEqual(1, result)

        tested.billing_line_items = BillingLineItemRecordSet([
            {'cpt': '99202', 'datetimeOfService': '2018-03-04T21:41:21.407046Z', },
            {'cpt': '99202', 'datetimeOfService': '2018-03-06T21:41:21.407046Z', },
            {'cpt': '99212', 'datetimeOfService': '2018-04-05T21:41:21.407046Z', },
            {'cpt': '99344', 'datetimeOfService': '2018-05-05T21:41:21.407046Z', },
            {'cpt': '99215', 'datetimeOfService': '2018-06-05T21:41:21.407046Z', },
            {'cpt': '99215', 'datetimeOfService': '2019-03-06T21:41:21.407046Z', },
        ])  # yapf: disable
        # incorrect ValueSet + billing item --> False
        result = tested.count_visit_within(timeframe, AnnualWellnessVisit)
        self.assertEqual(0, result)
        # correct ValueSet + billing item + date within the time frame --> False
        result = tested.count_visit_within(timeframe, OfficeVisit)
        self.assertEqual(3, result)
        result = tested.count_visit_within(
            timeframe, AnnualWellnessVisit | OfficeVisit | HomeHealthcareServices)
        self.assertEqual(4, result)

        # date outside the time frame
        tested.billing_line_items = BillingLineItemRecordSet([{
            'id': 92,
            'created': '2018-02-05T21:41:21.407046Z',
            'datetimeOfService': '2018-02-05T21:41:21.407046Z',
            'cpt': '99202',
            'units': 1
        }])
        result = tested.has_visit_within(timeframe, OfficeVisit)
        self.assertFalse(result)

    def test_as_dict(self):
        fields = [
            'administrative_documents',
            'allergy_intolerances',
            'billing_line_items',
            'conditions',
            'imaging_reports',
            'immunizations',
            'instructions',
            'interviews',
            'lab_reports',
            'medications',
            'procedures',
            'protocol_overrides',
            'prescriptions',
            'reason_for_visits',
            'referrals',
            'referral_reports',
            'upcoming_appointments',
            'upcoming_appointment_notes',
            'vital_signs',
        ]
        tested = self.load_patient('full')
        the_dict = tested.as_dict()

        self.assertTrue('patient_key' in the_dict)
        self.assertEqual('full', the_dict['patient_key'])
        self.assertTrue('patient' in the_dict)
        for field in fields:
            self.assertTrue(field in the_dict, 'patient.as_dict misses the field %s' % field)

    def test_print(self):
        tested = self.load_patient('full')
        f = io.StringIO()
        with redirect_stdout(f):
            tested.print()
        s = f.getvalue()
        self.assertGreater(len(s), 100)
        self.assertEqual('{\n  "patient_key": "full",\n', s[:27])

        # --- helpers

    def helper_age(self, year, month, day) -> float:
        bday = datetime(year=year, month=month, day=day)
        cday = datetime.today()
        age = 0
        if bday < cday:
            while bday.year < cday.year:
                age += 1
                bday = datetime(year=year + age, month=month, day=day)

            nominator = 0
            denominator = 0
            if bday < cday:
                nextyear = datetime(year=year + age + 1, month=month, day=day)
                while bday < nextyear:
                    bday += timedelta(days=1)
                    denominator += 1
                    if bday < cday:
                        nominator += 1
            else:
                age = age - 1
                nextyear = datetime(year=year + age, month=month, day=day)
                while nextyear < bday:
                    nextyear += timedelta(days=1)
                    denominator += 1
                    if nextyear < cday:
                        nominator += 1
            age += nominator / denominator

        return age



from protocols import events
from protocols.protocol import ClinicalQualityMeasure
from protocols.value_set.v2021 import (
    AllergyToBetaBlockerTherapy, Arrhythmia, Asthma, AtrioventricularBlock, BetaBlockerTherapy,
    BetaBlockerTherapyForLvsd, BetaBlockerTherapyIngredient, Bradycardia, CardiacPacerInSitu,
    CareServicesInLongTermResidentialFacility, CoronaryArteryDiseaseNoMi, EjectionFraction,
    Ethnicity, HeartRate, HomeHealthcareServices, Hypotension, IntoleranceToBetaBlockerTherapy,
    LeftVentricularSystolicDysfunction, ModerateOrSevereLvsd, MyocardialInfarction,
    NursingFacilityVisit, OfficeVisit, OncAdministrativeSex, OutpatientConsultation,
    PatientProviderInteraction, Payer, Race)


class ClinicalQualityMeasure145v9(ClinicalQualityMeasure):
    """
    Coronary Artery Disease (CAD): Beta-Blocker Therapy-Prior Myocardial Infarction (MI) or Left
    Ventricular Systolic Dysfunction (LVEF <40%)

    Description: Percentage of patients aged 18 years and older with a diagnosis of coronary artery
    disease seen within a 12-month period who also have a prior MI or a current or prior LVEF <40%
    who were prescribed beta-blocker therapy

    Definition: Prescribed may include prescription given to the patient for beta-blocker therapy
    at one or more visits in the measurement period OR patient already taking beta-blocker therapy
    as documented in current medication list.

    Prior Myocardial Infarction (MI) for denominator 2 is limited to those occurring within the
    past 3 years.

    Rationale: For patients with coronary artery disease (CAD), beta-blockers are recommended for 3
    years after myocardial infarction or acute coronary syndrome. Beta-blockers, particularly
    carvedilol, metoprolol succinate, or bisoprolol which have been shown to reduce risk of death,
    are recommended indefinitely for patients with CAD and LV systolic dysfunction. These agents
    have proven efficacy in reducing angina onset and improving the ischemic threshold during
    exercise. In patients who have suffered an MI, beta-blockers significantly reduce deaths and
    recurrent MIs (ACCF/AHA/ACP/AATS/PCNA/SCAI/STS, 2012).

    Nonadherence to cardioprotective medications is prevalent among outpatients with CAD and can be
    associated with a broad range of adverse outcomes, including all-cause and cardiovascular
    mortality, cardiovascular hospitalizations, and the need for revascularization procedures
    (ACC/AHA, 2002).

    This measure is intended to promote beta-blocker usage in select patients with CAD.

    Guidance: Beta-blocker therapy:
    - For patients with prior MI, beta-blocker therapy includes any agent within the beta-blocker
    drug class. As of 2015, no recommendations or evidence are cited in current stable ischemic
    heart disease guidelines for preferential use of specific agents
    - For patients with prior LVEF <40%, beta-blocker therapy includes the following: bisoprolol,
    carvedilol, or sustained release metoprolol succinate

    The requirement of two or more visits is to establish that the eligible professional or
    eligible clinician has an existing relationship with the patient.

    A range value should satisfy the logic requirement for 'Ejection Fraction' as long as the
    ranged observation value clearly meets the less than 40% threshold noted in the denominator
    logic. A range that is inclusive of or greater than 40% would not meet the measure requirement.

    If a patient has had a myocardial infarction (MI) within the past 3 years and a current or
    prior LVEF < 40% (or moderate or severe LVSD), the patient should only be counted in Population
    Criteria 1.

    This eCQM is a patient-based measure.

    This version of the eCQM uses QDM version 5.5. Please refer to the eCQI resource center
    (https://ecqi.healthit.gov/qdm) for more information on the QDM.

    More information: https://ecqi.healthit.gov/ecqm/measures/cms145v9
    """

    title = 'Coronary Artery Disease (CAD): Beta-Blocker Therapy-Prior Myocardial Infarction (MI) or Left Ventricular Systolic Dysfunction (LVEF <40%)'

    identifiers = ['CMS145v9']

    types = ['CQM']

    responds_to_event_types = [
        events.HEALTH_MAINTENANCE,
    ]

    authors = [
        'American Medical Association (AMA)',
        'PCPI(R) Foundation (PCPI[R])',
    ]

    references = [
        'Fihn, S. D., Gardin, J. M., Abrams, J., Berra, K., Blankenship, J. C., Dallas, A. P., … Williams, S. V. (2012). 2012 ACCF/AHA/ACP/AATS/PCNA/SCAI/STS Guideline for the Diagnosis and Management of Patients With Stable Ischemic Heart Disease. Circulation, 126(25), e354–e471. doi: 10.1161/cir.0b013e318277d6a0',
        'Gibbons, R. J., Abrams, J., Chatterjee, K., Daley, J., Deedwania, P.K, Douglas J.S., … Smith Jr., S.C. (2002). ACC/AHA 2002 Guideline Update for the Management of Patients with Chronic StableAangina: A Report of the American College of Cardiology/American Heart Association Task Force on Practice Guidelines (Committee to Update the 1999 Guidelines for the Management of Patients with Chronic Stable Angina). Circulation, 107(1), 149-158. doi:10.1161/01.CIR.0000047041.66447.29',
    ]

    funding_source = ''

    def in_initial_population(self):
        """
        Initial population: All patients aged 18 years and older with a diagnosis of coronary
        artery disease seen within a 12-month period
        """
        pass

    def in_denominator(self):
        """
        Denominator: Equals Initial Population who also have prior (within the past 3 years) MI or
        a current or prior LVEF <40%

        Exclusions: None

        Exceptions: Documentation of medical reason(s) for not prescribing beta-blocker therapy
        (e.g., allergy, intolerance, other medical reasons).

        Documentation of patient reason(s) for not prescribing beta-blocker therapy (e.g., patient
        declined, other patient reasons).

        Documentation of system reason(s) for not prescribing beta-blocker therapy (e.g., other
        reasons attributable to the health care system).
        """
        pass

    def in_numerator(self):
        """
        Numerator: Patients who were prescribed beta-blocker therapy

        Exclusions: Not Applicable
        """
        pass


    def compute_results(self):
        """
        Clinical recommendation: Beta-blocker therapy should be started and continued for 3 years
        in all patients with normal LV function after MI or ACS (Class I, Level of Evidence: B)
        (ACCF/AHA/ACP/AATS/PCNA/SCAI/STS, 2012).

        Beta-blocker therapy should be used in all patients with LV systolic dysfunction (EF <=
        40%) with heart failure or prior MI, unless contraindicated. (Use should be limited to
        carvedilol, metoprolol succinate, or bisoprolol, which have been shown to reduce risk of
        death.) (Class I, Level of Evidence: A) (ACCF/AHA/ACP/AATS/PCNA/SCAI/STS, 2012).
        """
        pass



from protocols import events
from protocols.protocol import ClinicalQualityMeasure
from protocols.value_set.v2021 import (
    AnnualWellnessVisit, AudiologyVisit, CareServicesInLongTermResidentialFacility,
    DischargeServicesNursingFacility, EncounterInpatient, Ethnicity, FallsScreening,
    HomeHealthcareServices, HospiceCareAmbulatory, NursingFacilityVisit, OfficeVisit,
    OncAdministrativeSex, OphthalmologicalServices, Payer,
    PreventiveCareServicesEstablishedOfficeVisit18AndUp,
    PreventiveCareServicesIndividualCounseling, PreventiveCareServicesInitialOfficeVisit18AndUp,
    Race)


class ClinicalQualityMeasure139v9(ClinicalQualityMeasure):
    """
    Falls: Screening for Future Fall Risk

    Description: Percentage of patients 65 years of age and older who were screened for future fall
    risk during the measurement period

    Definition: Screening for Future Fall Risk: Assessment of whether an individual has experienced
    a fall or problems with gait or balance.  A specific screening tool is not required for this
    measure, however potential screening tools include the Morse Fall Scale and the timed Get-Up-
    And-Go test.

    Fall: A sudden, unintentional change in position causing an individual to land at a lower
    level, on an object, the floor, or the ground, other than as a consequence of sudden onset of
    paralysis, epileptic seizure, or overwhelming external force.

    Rationale: As the leading cause of both fatal and nonfatal injuries for older adults, falls are
    one of the most common and significant health issues facing people aged 65 years or older
    (Schneider, Shubert and Harmon, 2010). Moreover, the rate of falls increases with age (Dykes et
    al., 2010). Older adults are five times more likely to be hospitalized for fall-related
    injuries than any other cause-related injury. It is estimated that one in every three adults
    over 65 will fall each year (Centers for Disease Control and Prevention 2015). In those over
    age 80, the rate of falls increases to fifty percent (Doherty et al., 2009). Falls are also
    associated with substantial cost and resource use, approaching $30,000 per fall hospitalization
    Woolcott et al., 2011). Identifying at-risk patients is the most important part of management,
    as applying preventive measures in this vulnerable population can have a profound effect on
    public health (al-Aama, 2011). Family physicians have a pivotal role in screening older
    patients for risk of falls, and applying preventive strategies for patients at risk (al-Aama,
    2011).

    Guidance: This eCQM is a patient-based measure.

    This version of the eCQM uses QDM version 5.5.  Please refer to the eCQI resource center
    (https://ecqi.healthit.gov/qdm) for more information on the QDM.

    More information: https://ecqi.healthit.gov/ecqm/measures/cms139v9
    """

    title = 'Falls: Screening for Future Fall Risk'

    identifiers = ['CMS139v9']

    types = ['CQM']

    responds_to_event_types = [
        events.HEALTH_MAINTENANCE,
    ]

    authors = [
        'National Committee for Quality Assurance',
        'American Medical Association (AMA)',
        'PCPI(R) Foundation (PCPI[R])',
    ]

    references = [
        'al-Aama, T. 2011. "Falls in the Elderly: Spectrum and Prevention." Can Fam Physician 57(7):771-6.',
        'American Geriatrics Society and British Geriatric Society. (2010) Prevention of Falls in Older Persons Clinical Practice Guidelines. Accessed June 14, 2018. Available at https://www.archcare.org/sites/default/files/pdf/2010-prevention-of-falls-in-older-persons-ags-and-bgs-clinical-practice-guideline.pdf',
        'Centers for Disease Control and Prevention. 2015. "Important Facts about Falls" (December 14, 2015) http://www.cdc.gov/HomeandRecreationalSafety/Falls/adultfalls.html',
        'Doherty, M., and J. Crossen-Sills. 2009. "Fall Risk: Keep your patients in balance." The Nurse Practitioner: The American Journal of Primary Health Care 34(12):46-51.',
        'Dykes, P.C., D.L. Carroll DL, A. Hurley A, S. Lipsitz S, A. Benoit A, F. Chang F, S. Meltzer S, R. Tsurikova R, L. Zuyov L, B. Middleton B. 2010. "Fall Prevention in Acute Care Hospitals: A Randomized Trial." JAMA . 2010;304(17):1912-1918.',
        'Schneider, E.C., T.E. Shubert, and K.J. Harmon. 2010. "Addressing the Escalating Public Health Issue of Falls Among Older Adults." NC Med J 71(6):547-52.',
        'Woolcott, J.C., K.M. Khan, S. Mitrovic, A.H. Anis, C.A. Marra. 2011. "The Cost of Fall Related Presentations to the ED: A Prospective, In-Person, Patient-Tracking Analysis of Health Resource Utilization." Osteporos Int [Epub ahead of print].',
    ]

    funding_source = ''

    def in_initial_population(self):
        """
        Initial population: Patients aged 65 years and older with a visit during the measurement
        period
        """
        pass

    def in_denominator(self):
        """
        Denominator: Equals Initial Population

        Exclusions: Exclude patients whose hospice care overlaps the measurement period.

        Exceptions: None
        """
        return self.in_initial_population()

    def in_numerator(self):
        """
        Numerator: Patients who were screened for future fall risk at least once within the
        measurement period

        Exclusions: Not Applicable
        """
        pass


    def compute_results(self):
        """
        Clinical recommendation: All older persons who are under the care of a heath professional
        (or their caregivers) should be asked at least once a year about falls.  (AGS/BGS/AAOS
        2010)

        Older persons who present for medical attention because of a fall, report recurrent falls
        in the past year, or demonstrate abnormalities of gait and/or balance should have a fall
        evaluation performed.  This evaluation should be performed by a clinician with appropriate
        skills and experience, which may necessitate referral to a specialist (e.g., geriatrician).
        (AGS/BGS/AAOS 2010)
        """
        pass



from protocols import events
from protocols.protocol import ClinicalQualityMeasure
from protocols.value_set.v2021 import (
    AlcoholAndDrugDependence, AlcoholAndDrugDependenceTreatment, DetoxificationVisit,
    DischargeServicesHospitalInpatient, DischargeServicesHospitalInpatientSameDayDischarge,
    EmergencyDepartmentVisit, EncounterInpatient, Ethnicity, HospiceCareAmbulatory,
    HospitalInpatientVisitInitial, HospitalObservationCareInitial, OfficeVisit,
    OncAdministrativeSex, OpiateAntagonists, Payer, PsychVisitPsychotherapy, Race,
    TelehealthServices)


class ClinicalQualityMeasure137v9(ClinicalQualityMeasure):
    """
    Initiation and Engagement of Alcohol and Other Drug Dependence Treatment

    Description: Percentage of patients 13 years of age and older with a new episode of alcohol or
    other drug abuse or (AOD) dependence who received the following. Two rates are reported.

    a. Percentage of patients who initiated treatment including either an intervention or
    medication for the treatment of AOD abuse or dependence within 14 days of the diagnosis
    b. Percentage of patients who engaged in ongoing treatment including two additional
    interventions or a medication for the treatment of AOD abuse or dependence within 34 days of
    the initiation visit. For patients who initiated treatment with a medication, at least one of
    the two engagement events must be a treatment intervention.

    Definition: The initiation visit is the first visit for alcohol or other drug dependence
    treatment within 14 days after a diagnosis of alcohol or other drug dependence.

    Treatment includes inpatient AOD admissions, outpatient visits, intensive outpatient encounters
    or partial hospitalization.

    The Intake Period: January 1-November 14 of the measurement year. The Intake Period is used to
    capture new episodes of Alcohol or Drug Dependence. The November 14 cut-off date ensures that
    all services can occur before the measurement period ends.

    Rationale: There are more deaths, illnesses and disabilities from substance abuse than from any
    other preventable health condition. In 2017, 19.7 million individuals in the U.S. age 12 or
    older (approximately 8 percent of the population) were classified as having an SUD within the
    past year (SAMHSA, 2018). Despite the high prevalence of SUD in the U.S., only 20 percent of
    individuals with SUD receive any substance use treatment and only 13 percent receive treatment
    in a specialty SUD program (SAMHSA, 2018a).

    Guidance: The new episode of alcohol and other drug dependence should be the first episode of
    the measurement period that is not preceded in the 60 days prior by another episode of alcohol
    or other drug dependence.

    This eCQM is a patient-based measure.

    This version of the eCQM uses QDM version 5.5.  Please refer to the eCQI resource center
    (https://ecqi.healthit.gov/qdm) for more information on the QDM.

    More information: https://ecqi.healthit.gov/ecqm/measures/cms137v9
    """

    title = 'Initiation and Engagement of Alcohol and Other Drug Dependence Treatment'

    identifiers = ['CMS137v9']

    types = ['CQM']

    responds_to_event_types = [
        events.HEALTH_MAINTENANCE,
    ]

    authors = [
        'National Committee for Quality Assurance',
    ]

    references = [
        'American Psychiatric Association: Work Group on Substance Use Disorders. (2006). Practice Guideline for the Treatment of Patients with Substance Use Disorders Second Edition. American Psychiatric Association (APA); 2006 Aug. 276 pg. [1789 references]. Retrieved from https://psychiatryonline.org/pb/assets/raw/sitewide/practice_guidelines/guidelines/substanceuse.pdf',
        'Michigan Quality Improvement Consortium. (2017). Screening, diagnosis, and referral for substance use disorders. Southfield, (MI): Michigan Quality Improvement Consortium. Retrieved from http://www.mqic.org/pdf/mqic_screening_diagnosis_and_referral_for_substance_use_disorders_cpg.pdf',
        'Schneider Institute for Health Policy, Brandeis University. 2001. Substance Abuse: The Nation\'s Number One Health Problem. Princeton: Robert Wood Johnson Foundation.',
        'Department of Veteran Affairs, Department of Defense. (2015). VA/DoD Clinical Practice Guideline for the Management of Substance Use Disorders. Washington DC: Department of Veterans Affairs, Department of Defense.',
        'Kampman, K., Jarvis, M. (2015). American Society of Addiction Medicine (ASAM) National Practice Guideline for the Use of Medications in the Treatment of Addiction Involving Opioid Use. Journal of Addiction Medicine; 9(5): 358–367. DOI: 10.1097/ADM.0000000000000166.',
        'Pietras, S., M. Azur, J. Brown. 2015. Review of Medication-Assisted Treatment Guidelines and Measures for Opioid and Alcohol Use. (ASPE). Report prepared by Mathematica Policy Research for the U.S. Department of Health and Human Services, Assistant Secretary for Planning and Evaluation, Office of Disability, Aging and Long-Term Care Policy. https://aspe.hhs.gov/sites/default/files/pdf/205171/MATguidelines.pdf (November 16, 2016)',
        'Reus, V. et al. (2018). Practice Guideline for the Pharmacological Treatment of Patients with Alcohol Use Disorder. American Journal of Psychiatry, 175(1), 86-90. doi:10.1176/appi.ajp.2017.1750101',
        'Substance Abuse and Mental Health Services Administration (SAMHSA). 2018a. “Key Substance Use and Mental Health Indicators in the United States: Results from the 2017 National Survey on Drug Use and Health.” Retrieved from https://www.samhsa.gov/data/sites/default/files/cbhsq-reports/NSDUHFFR2017/NSDUHFFR2017.htm#sud11',
    ]

    funding_source = ''

    def in_initial_population(self):
        """
        Initial population: Patients age 13 years of age and older who were diagnosed with a new
        episode of alcohol, opioid, or other drug abuse or dependency during a visit between
        January 1 and November 14 of the measurement period
        """
        pass

    def in_denominator(self):
        """
        Denominator: Equals Initial Population

        Exclusions: Exclude patients with a previous active diagnosis of alcohol, opioid or other
        drug abuse or dependence in the 60 days prior to the first episode of alcohol or drug
        dependence.

        Exclude patients whose hospice care overlaps the measurement period.

        Exceptions: None
        """
        return self.in_initial_population()

    def in_numerator(self):
        """
        Numerator: Numerator 1: Initiation of treatment includes either an intervention or
        medication for the treatment of AOD abuse or dependence within 14 days of the diagnosis.

        Numerator 2: Engagement in ongoing treatment includes two additional interventions or a
        medication for the treatment of AOD abuse or dependence within 34 days of the initiation
        visit. For patients who initiated treatment with a medication, at least one of the two
        engagement events must be a treatment intervention (i.e., engagement for these members
        cannot be satisfied with medication treatment alone).

        Exclusions: Not Applicable
        """
        pass


    def compute_results(self):
        """
        Clinical recommendation: American Psychiatric Association (2018)
        * Patients with alcohol use disorder should have a documented comprehensive and person-
        centered treatment plan that includes evidence-based nonpharmacological and pharmacological
        treatments. [1C]
        * Naltrexone or acamprosate be offered to patients with moderate to severe alcohol use
        disorder who have a goal of reducing alcohol consumption or achieving abstinence, prefer
        pharmacotherapy or have not responded to nonpharmacological treatments alone, and have no
        contraindications to the use of these medications. [1B]
        * Disulfiram should be offered to patients with moderate to severe alcohol use disorder who
        have a goal of achieving abstinence, prefer disulfiram or are intolerant to or have not
        responded to naltrexone and acamprosate, are capable of understanding the risks of alcohol
        consumption while taking disulfiram, and have no contraindications to the use of this
        medication. [2C]
        * Topiramate or gabapentin be offered to patients with moderate to severe alcohol use
        disorder who have a goal of reducing alcohol consumption or achieving abstinence, prefer
        topiramate or gabapentin or are intolerant to or have not responded to naltrexone and
        acamprosate, and have no contraindications to the use of these medications. [2C]

        American Psychiatric Association (2006)
        * Because many substance use disorders are chronic, patients usually require long-term
        treatment, although the intensity and specific components of treatment may vary over time
        [I rating].
        * It is important to intensify the monitoring for substance use during periods when the
        patient is at a high risk of relapsing, including during the early stages of treatment,
        times of transition to less intensive levels of care, and the first year after active
        treatment has ceased [I rating].
        * Outpatient treatment of substance use disorders is appropriate for patients whose
        clinical condition or environmental circumstances do not require a more intensive level of
        care [I rating]. As in other treatment settings, a comprehensive approach is optimal,
        using, where indicated, a variety of psychotherapeutic and pharmacological interventions
        along with behavioral monitoring [I rating ].
        * Disulfiram is also recommended for patients with alcohol dependence [II rating].
        * Naltrexone, injectable naltrexone, acamprosate, a y-aminobutyric acid (GABA) are
        recommended for patients with alcohol dependence [I rating]. Disulfiram is also recommended
        for patients with alcohol dependence [II rating].
        * Methadone and buprenorphine are recommended for patients with opioid dependence [I
        rating].
        * Naltrexone is an alternative strategy [I rating].

        American Society of Addiction Medicine (2015)
        * Methadone and buprenorphine are recommended for opioid use disorder treatment and
        withdrawal management.
        * Naltrexone (oral; extended-release injectable) is recommended for relapse prevention.

        Michigan Quality Improvement Consortium (2017)
        *Patients with substance use disorder or risky substance use: Patient Education and Brief
        Intervention by PCP or Trained Staff (e.g. RN, MSW)
        *If diagnosed with substance use disorder or risky substance use, initiate an intervention
        within 14 days.
        *Frequent follow-up is helpful to support behavior change; preferably 2 visits within 30
        days.
        *Refer to a substance abuse health specialist, an addiction physician specialist, or a
        physician experienced in pharmacologic management of addiction.

        Department of Veterans Affairs/Department of Defense (2015)
        * Offer referral to specialty SUD care for addiction treatment if based on willingness to
        engage. [B]
        * For patients with moderate-severe alcohol use disorder, we recommend: Acamprosate,
        Disulfiram, Naltrexone- oral or extended release, or Topiramate. [A]
        * Medications should be offered in combined with addiction-focused counseling. offering one
        or more of the following interventions considering patient preference and provider
        training/competence: Behavioral Couples Therapy for alcohol use disorder, Cognitive
        Behavioral Therapy for substance use disorders, Community Reinforcement Approach,
        Motivational Enhancement Therapy, 12-Step Facilitation. [A]
        * For patients with opioid use disorder we recommend buprenorphine/naloxone or methadone in
        an Opioid Treatment Program. For patients for whom agonist treatment is contraindicated,
        unacceptable, unavailable, or discontinued, we recommend extended-release injectable
        naltrexone. [A]
        * For patients initiated in an intensive phase of outpatient or residential treatment,
        recommend ongoing systematic relapse prevention efforts or recovery support, individualized
        on the basis of treatment response. [A]
        """
        pass



from protocols import events
from protocols.protocol import ClinicalQualityMeasure
from protocols.value_set.v2021 import (
    AnkylosingSpondylitis, AromataseInhibitors, AverageNumberOfDrinksPerDrinkingDay, BmiRatio,
    ChronicLiverDisease, ChronicMalnutrition, CushingsSyndrome,
    DxaDualEnergyXrayAbsorptiometryScan, EhlersDanlosSyndrome, EndStageRenalDisease, Ethnicity,
    Female, GlucocorticoidsOralOnly, HistoryOfHipFractureInParent, Hyperparathyroidism,
    Hyperthyroidism, Lupus, MalabsorptionSyndromes, MarfansSyndrome, OfficeVisit,
    OncAdministrativeSex, OsteogenesisImperfecta, Osteopenia, Osteoporosis, OsteoporoticFractures,
    OutpatientConsultation, Payer, PreventiveCareServicesEstablishedOfficeVisit18AndUp,
    PreventiveCareServicesInitialOfficeVisit18AndUp, PreventiveCareServicesOther,
    PsoriaticArthritis, Race, RheumatoidArthritis, TobaccoUseScreening, Type1Diabetes, White)


class ClinicalQualityMeasure249v3(ClinicalQualityMeasure):
    """
    Appropriate Use of DXA Scans in Women Under 65 Years Who Do Not Meet the Risk Factor Profile
    for Osteoporotic Fracture

    Description: Percentage of female patients 50 to 64 years of age without select risk factors
    for osteoporotic fracture who received an order for a dual-energy x-ray absorptiometry (DXA)
    scan during the measurement period

    Definition: The measure allows for clinicians to use the Fracture Risk Assessment Tool
    (FRAX[R]) to calculate 10-year absolute fracture risk. The FRAX was developed by the World
    Health Organization in 2008 to evaluate a patient's 10-year probability of hip fracture and
    major osteoporotic fracture (clinical spine, forearm, hip, or shoulder fracture). It is
    applicable to people aged 40-90 years.

    Rationale: This measure is expected to increase recording of patient risk for fracture data and
    decrease the amount of inappropriate DXA scans. Current osteoporosis guidelines recommend using
    bone measurement testing to assess osteoporosis risk in women 65 years and older. In
    postmenopausal women younger than age 65, guidelines recommend using a formal clinical risk
    assessment tool to establish a patient's risk for osteoporosis, in order to determine whether
    to screen a patient for osteoporosis using bone measurement testing. Clinical information, such
    as age, body mass index (BMI), parental hip fracture history, and smoking and alcohol use, can
    be used to determine a woman's fracture risk (U.S. Preventive Services Task Force [USPSTF],
    2018). Additionally, there are potentially avoidable harms associated with screening for
    osteoporosis in general, including exposure to radiation, false positive exams, and resulting
    side effects from unnecessary osteoporosis medications, which add costs to an already burdened
    health care system (Lim, Hoeksema, & Sherin, 2009).

    Guidance: There are two ways that a patient can be excluded from the measure:
    1. The patient has a specific number of "combination" risk factors (the number of risk factors
    varies by age).
    2. The patient has one or more of the "independent" risk factors, including a 10-year
    probability of major osteoporotic fracture of 8.4 percent or higher as determined by the FRAX.

    This eCQM is a patient-based measure.

    This version of the eCQM uses QDM version 5.5.  Please refer to the eCQI resource center
    (https://ecqi.healthit.gov/qdm) for more information on the QDM.

    More information: https://ecqi.healthit.gov/ecqm/measures/cms249v3
    """

    title = 'Appropriate Use of DXA Scans in Women Under 65 Years Who Do Not Meet the Risk Factor Profile for Osteoporotic Fracture'

    identifiers = ['CMS249v3']

    types = ['CQM']

    responds_to_event_types = [
        events.HEALTH_MAINTENANCE,
    ]

    authors = [
        'National Committee for Quality Assurance',
    ]

    references = [
        'Lim, L. S., Hoeksema, L. J., & Sherin, K. (2009). Screening for osteoporosis in the adult U.S. population: ACPM position statement on preventive practice. American Journal of Preventive Medicine, 36(4), 366-375.',
        'National Institute for Health and Clinical Excellence. (2017). Osteoporosis: Fragility fracture risk. Retrieved from https://www.nice.org.uk/guidance/cg146/chapter/1-Guidance',
        'U.S. Preventive Services Task Force, Curry S. J., Krist, A. H., et al. (2018). Screening for osteoporosis to prevent fractures: U.S. Preventive Services Task Force recommendation statement. JAMA, 319(24), 2521-2531.',
    ]

    funding_source = ''

    def in_initial_population(self):
        """
        Initial population: Female patients ages 50 to 64 years with an encounter during the
        measurement period
        """
        pass

    def in_denominator(self):
        """
        Denominator: Equals Initial Population

        Exclusions: Exclude patients with a combination of risk factors (as determined by age) or
        one of the independent risk factors

        Ages: 50-54 (>=4 combination risk factors) or 1 independent risk factor
        Ages: 55-59 (>=3 combination risk factors) or 1 independent risk factor
        Ages: 60-64 (>=2 combination risk factors) or 1 independent risk factor

        COMBINATION RISK FACTORS [The following risk factors are all combination risk factors; they
        are grouped by when they occur in relation to the measurement period]:

        The following risk factors may occur any time in the patient's history but must be active
        during the measurement period:
        White (race)
        BMI <= 20 kg/m2 (must be the first BMI of the measurement period)
        Smoker (current during the measurement period)
        Alcohol consumption (> two units per day (one unit is 12 oz. of beer, 4 oz. of wine, or 1
        oz. of liquor))

        The following risk factor may occur any time in the patient's history and must not start
        during the measurement period:
        Osteopenia

        The following risk factors may occur at any time in the patient's history or during the
        measurement period:
        Rheumatoid arthritis
        Hyperthyroidism
        Malabsorption Syndromes: celiac disease, inflammatory bowel disease, ulcerative colitis,
        Crohn's disease, cystic fibrosis, malabsorption
        Chronic liver disease
        Chronic malnutrition

        The following risk factors may occur any time in the patient's history and do not need to
        be active at the start of the measurement period:
        Documentation of history of hip fracture in parent
        Osteoporotic fracture
        Glucocorticoids (>= 5 mg/per day) [cumulative medication duration >= 90 days]

        INDEPENDENT RISK FACTORS (The following risk factors are all independent risk factors; they
        are grouped by when they occur in relation to the measurement period):

        The following risk factors may occur at any time in the patient's history and must not
        start during the measurement period:
        Osteoporosis

        The following risk factors may occur at any time in the patient's history prior to the
        start of the measurement period, but do not need to be active during the measurement
        period:
        Gastric bypass
        FRAX[R] ten-year probability of all major osteoporosis related fracture >= 8.4 percent
        Aromatase inhibitors

        The following risk factors may occur at any time in the patient's history or during the
        measurement period:
        Type I Diabetes
        End stage renal disease
        Osteogenesis imperfecta
        Ankylosing spondylitis
        Psoriatic arthritis
        Ehlers-Danlos syndrome
        Cushing's syndrome
        Hyperparathyroidism
        Marfan syndrome
        Lupus

        Exceptions: None
        """
        return self.in_initial_population()

    def in_numerator(self):
        """
        Numerator: Female patients who received an order for at least one DXA scan in the
        measurement period

        Exclusions: Not Applicable
        """
        pass


    def compute_results(self):
        """
        Clinical recommendation: USPSTF:
        "The USPSTF recommends screening for osteoporosis with bone measurement testing to prevent
        osteoporotic fractures in women 65 years and older." This is a B recommendation.

        "The USPSTF concludes that the current evidence is insufficient to assess the balance of
        benefits and harms of screening for osteoporosis to prevent osteoporotic fractures in men."
        This is an I statement.

        "Several tools are available to assess osteoporosis risk: the Simple Calculated
        Osteoporosis Risk Estimate (SCORE; Merck), Osteoporosis Risk Assessment Instrument (ORAI),
        Osteoporosis Index of Risk (OSIRIS), and the Osteoporosis Self-Assessment Tool (OST). These
        tools seem to perform similarly and are moderately accurate at predicting osteoporosis. The
        FRAX tool (University of Sheffield), which assesses a person's 10-year risk of fracture, is
        also a commonly used tool."

        "The USPSTF recommends screening for osteoporosis with bone measurement testing to prevent
        osteoporotic fractures in postmenopausal women younger than 65 years who are at increased
        risk of osteoporosis, as determined by a formal clinical risk assessment tool." This is a B
        recommendation.
        "For postmenopausal women younger than 65 years who have at least 1 risk factor, a
        reasonable approach to determine who should be screened with bone measurement testing is to
        use a clinical risk assessment tool."

        "Because the benefits of treatment are greater in persons at higher risk of fracture, one
        approach is to perform bone measurement testing in postmenopausal women younger than 65
        years who have a 10-year FRAX risk of major osteoporotic fracture (MOF) (without DXA)
        greater than that of a 65-year-old white woman without major risk factors. For example, in
        the United States, a 65-year-old white woman of mean height and weight without major risk
        factors has a 10-year FRAX risk of MOF of 8.4%."

        The National Institute for Health and Clinical Excellence’s Osteoporosis Guidelines:
        "Consider assessment of fracture risk:
        a. in all women aged 65 years and over and all men aged 75 years and over
        b. in women aged under 65 years and men aged under 75 years in the presence of risk
        factors, for
        example:
        *previous fragility fracture
        *current use or frequent/ recent use of oral or systemic glucocorticoids
        *history of falls
        *family history of hip fracture
        *other causes of secondary osteoporosis
        *low body mass index (BMI) (less than 18.5 kg/m2)
        *smoking
        *alcohol intake of more than 14 units per week for women and more than 21 units per week
        for men."

        "Do not routinely assess fracture risk in people aged under 50 years unless they have major
        risk
        factors (for example, current or frequent/recent use of oral or systemic glucocorticoids,
        untreated
        premature menopause or previous fragility fracture), because they are unlikely to be at
        high risk."

        "Estimate absolute risk when assessing risk of fracture (for example, the predicted risk of
        major osteoporotic or hip fracture over 10 years, expressed as a percentage."

        "Use either FRAX (without a bone mineral density [BMD] value if a dual energy X-ray
        absorptiometry [DXA] scan has not previously been undertaken) or QFracture, within their
        allowed age ranges, to estimate 10-year predicted absolute fracture risk when assessing
        risk of fracture."

        "Do not routinely measure BMD to assess fracture risk without prior assessment using FRAX
        (without a BMD value) or QFracture."

        "Take into account that risk assessment tools may underestimate fracture risk in certain
        circumstances, for example if a person:
        *has a history of multiple fractures
        *has had previous vertebral fracture(s)
        *has a high alcohol intake
        *is taking high-dose oral or high-dose systemic glucocorticoids (more than 7.5 mg
        prednisolone
        or equivalent per day for 3 months or longer)
        *has other causes of secondary osteoporosis."
        """
        pass
